@@ -246,8 +246,6 @@ public class Main {
         InputStream in   = config.getScriptSource();
         String filename  = config.displayedFileName();
 
-        doProcessArguments(in);
-
         Ruby _runtime;
 
         if (DripMain.DRIP_RUNTIME != null) {
@@ -435,7 +433,7 @@ public class Main {
         } catch (RaiseException re) {
             if (re.getException().getMetaClass().getBaseName().equals("SyntaxError")) {
                 context.setErrorInfo($ex);
-                config.getError().println("SyntaxError in " + re.getException().message(runtime.getCurrentContext()));
+                config.getError().println("SyntaxError in " + re.getException().message(context));
                 return false;
             }
             throw re;
