@@ -163,6 +163,7 @@ public class RubyBigDecimal extends RubyNumeric {
         this.isNaN = false;
         this.infinitySign = 0;
         this.zeroSign = 0;
+        this.value = BigDecimal.ZERO;
     }
 
     public RubyBigDecimal(Ruby runtime, BigDecimal value) {
@@ -480,6 +481,9 @@ public class RubyBigDecimal extends RubyNumeric {
         if (value instanceof RubyFixnum || value instanceof RubyBignum) {
             return newInstance(context, context.runtime.getClass("BigDecimal"), value.asString());
         }
+        if ((value instanceof RubyRational) || (value instanceof RubyFloat)) {
+            return newInstance(context, context.runtime.getClass("BigDecimal"), value.asString(), RubyFixnum.newFixnum(context.runtime, RubyFloat.DIG));
+        }
         return cannotBeCoerced(context, value, must);
     }
 
@@ -508,33 +512,60 @@ public class RubyBigDecimal extends RubyNumeric {
         // precision can be no more than float digits
         if (mathContext.getPrecision() > RubyFloat.DIG + 1) throw runtime.newArgumentError("precision too large");
 
-        return new RubyBigDecimal(runtime, (RubyClass) recv, new BigDecimal(arg.getDoubleValue(), mathContext));
+        double dblVal = arg.getDoubleValue();
+        if(Double.isInfinite(dblVal) || Double.isNaN(dblVal)) throw runtime.newFloatDomainError("NaN");
+
+        return new RubyBigDecimal(runtime, (RubyClass) recv, new BigDecimal(dblVal, mathContext));
     }
 
     private static RubyBigDecimal newInstance(Ruby runtime, IRubyObject recv, RubyBignum arg, MathContext mathContext) {
         return new RubyBigDecimal(runtime, (RubyClass) recv, new BigDecimal(arg.getBigIntegerValue(), mathContext));
     }
 
-    private final static Pattern NUMBER_PATTERN = Pattern.compile("^([+-]?\\d*\\.?\\d*([eE][+-]?)?\\d*).*");
+    private final static Pattern NUMBER_PATTERN = Pattern.compile("^([+-]?\\d*\\.?\\d*([eE]?)([+-]?\\d*)).*");
 
     private static RubyBigDecimal newInstance(ThreadContext context, IRubyObject recv, IRubyObject arg, MathContext mathContext) {
         String strValue = arg.convertToString().toString().trim();
 
-        switch ( strValue.length() > 2 ? strValue.charAt(0) : ' ' ) { // do not case length == 1
+        int sign = 1;
+        switch ( strValue.length() > 0 ? strValue.charAt(0) : ' ' ) {
+            case '_' : return newZero(context.runtime, 1); // leading "_" are not allowed
             case 'N' :
                 if ( "NaN".equals(strValue) ) return newNaN(context.runtime);
+                break;
             case 'I' :
                 if ( "Infinity".equals(strValue) ) return newInfinity(context.runtime, 1);
+                break;
             case '-' :
                 if ( "-Infinity".equals(strValue) ) return newInfinity(context.runtime, -1);
+                sign = -1;
+                break;
             case '+' :
                 if ( "+Infinity".equals(strValue) ) return newInfinity(context.runtime, +1);
+                break;
         }
 
         // Convert String to Java understandable format (for BigDecimal).
         strValue = strValue.replaceFirst("[dD]", "E");                  // 1. MRI allows d and D as exponent separators
         strValue = strValue.replaceAll("_", "");                        // 2. MRI allows underscores anywhere
-        strValue = NUMBER_PATTERN.matcher(strValue).replaceFirst("$1"); // 3. MRI ignores the trailing junk
+
+        Matcher matcher = NUMBER_PATTERN.matcher(strValue);
+        strValue = matcher.replaceFirst("$1");                          // 3. MRI ignores the trailing junk
+
+        String exp = matcher.group(2); int idx;
+        if ( exp != null && ! exp.isEmpty() ) {
+            String expValue = matcher.group(3);
+            if (expValue.isEmpty() || expValue.equals("-") || expValue.equals("+")) {
+                strValue = strValue.concat("0");                        // 4. MRI allows 1E, 1E-, 1E+
+            }
+            else if (isExponentOutOfRange(expValue)) {
+                // Handle infinity (Integer.MIN_VALUE + 1) < expValue < Integer.MAX_VALUE
+                return newInfinity(context.runtime, sign);
+            }
+        }
+        else if ( ( idx = matcher.start(3) ) > 0 ) {
+            strValue = strValue.substring(0, idx); // ignored tail junk e.g. "5-6" -> "-6"
+        }
 
         BigDecimal decimal;
         try {
@@ -547,11 +578,38 @@ public class RubyBigDecimal extends RubyNumeric {
         }
 
         // MRI behavior: -0 and +0 are two different things
-        if (decimal.signum() == 0) return newZero(context.runtime, strValue.matches("^\\s*-.*") ? -1 : 1);
+        if (decimal.signum() == 0) return newZero(context.runtime, sign);
 
         return new RubyBigDecimal(context.runtime, (RubyClass) recv, decimal);
     }
 
+    private static boolean isExponentOutOfRange(final String expValue) {
+        int num = 0;
+        int sign = 1;
+        final int len = expValue.length();
+        final char ch = expValue.charAt(0);
+        if (ch == '-') {
+          sign = -1;
+        } else if (ch != '+') {
+            num = '0' - ch;
+        }
+        int i = 1;
+        final int max = (sign == 1) ? -Integer.MAX_VALUE : Integer.MIN_VALUE + 1;
+        final int multmax = max / 10;
+        while (i < len) {
+            int d = expValue.charAt(i++) - '0';
+            if (num < multmax) {
+                return true;
+            }
+            num *= 10;
+            if (num < (max + d)) {
+                return true;
+            }
+            num -= d;
+        }
+        return false;
+    }
+
     @Deprecated
     public static RubyBigDecimal newInstance(IRubyObject recv, IRubyObject[] args) {
         final ThreadContext context = recv.getRuntime().getCurrentContext();
@@ -566,7 +624,8 @@ public class RubyBigDecimal extends RubyNumeric {
     public static RubyBigDecimal newInstance(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         if ( context.runtime.is1_9() ) {
             if (arg instanceof RubyBigDecimal) return newInstance(context.runtime, recv, (RubyBigDecimal) arg);
-            if (arg instanceof RubyFloat || arg instanceof RubyRational) throw context.runtime.newArgumentError("can't omit precision for a rational");
+            if (arg instanceof RubyRational) throw context.runtime.newArgumentError("can't omit precision for a Rational.");
+            if (arg instanceof RubyFloat) throw context.runtime.newArgumentError("can't omit precision for a Float.");
             if (arg instanceof RubyFixnum) return newInstance(context.runtime, recv, (RubyFixnum) arg, MathContext.UNLIMITED);
             if (arg instanceof RubyBignum) return newInstance(context.runtime, recv, (RubyBignum) arg, MathContext.UNLIMITED);
         }
@@ -1300,9 +1359,6 @@ public class RubyBigDecimal extends RubyNumeric {
 
     @JRubyMethod(name = "coerce", required = 1)
     public RubyArray coerce(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFloat) {
-            return context.runtime.newArray(other, to_f());
-        }
         return context.runtime.newArray(getVpValue(context, other, true), this);
     }
 
@@ -1525,25 +1581,21 @@ public class RubyBigDecimal extends RubyNumeric {
 
     @JRubyMethod(name = "round", optional = 2)
     public IRubyObject round(ThreadContext context, IRubyObject[] args) {
-        int scale = args.length > 0 ? num2int(args[0]) : 0;
+        final int scale = args.length > 0 ? num2int(args[0]) : 0;
 
         // Special treatment for BigDecimal::NAN and BigDecimal::INFINITY
         //
         // If round is called without any argument, we should raise a
         // FloatDomainError. Otherwise, we don't have to call round ;
         // we can simply return the number itself.
-        if (scale == 0 && (isNaN() || isInfinity())) {
+        if (scale == 0 && isInfinity()) {
             StringBuilder message = new StringBuilder("Computation results to ");
-            message.append("'").append(callMethod(context, "to_s")).append("'");
+            message.append('\'').append(callMethod(context, "to_s")).append('\'');
 
-            // To be consistent with MRI's output
-            if (isNaN()) message.append("(Not a Number)");
-
-            throw getRuntime().newFloatDomainError(message.toString());
+            throw context.runtime.newFloatDomainError(message.toString());
         } else {
-            if (isNaN()) {
-                return newNaN(context.runtime);
-            } else if (isInfinity()) {
+            if (isNaN()) return newNaN(context.runtime);
+            if (isInfinity()) {
                 return newInfinity(context.runtime, infinitySign);
             }
         }
@@ -1558,19 +1610,14 @@ public class RubyBigDecimal extends RubyNumeric {
           // ...round to that digit
           BigDecimal rounded = normalized.setScale(0, mode);
           // ...and shift the result back to the left (multiply by 10**(abs(scale)))
-          bigDecimal = new RubyBigDecimal(getRuntime(), rounded.movePointLeft(scale));
+          bigDecimal = new RubyBigDecimal(context.runtime, rounded.movePointLeft(scale));
         } else {
-          bigDecimal = new RubyBigDecimal(getRuntime(), value.setScale(scale, mode));
+          bigDecimal = new RubyBigDecimal(context.runtime, value.setScale(scale, mode));
         }
         if (context.runtime.is1_8()) {
             return bigDecimal;
-        } else {
-            if (args.length == 0) {
-                return bigDecimal.to_int19();
-            } else {
-                return bigDecimal;
-            }
         }
+        return args.length == 0 ? bigDecimal.to_int19() : bigDecimal;
     }
 
     //this relies on the Ruby rounding enumerations == Java ones, which they (currently) all are
@@ -1606,31 +1653,11 @@ public class RubyBigDecimal extends RubyNumeric {
 
     @JRubyMethod(name = "sign")
     public IRubyObject sign() {
-        if (isNaN()) {
-            return getMetaClass().getConstant("SIGN_NaN");
-        }
+        if (isNaN()) return getMetaClass().getConstant("SIGN_NaN");
+        if (isInfinity()) return getMetaClass().getConstant(infinitySign < 0 ? "SIGN_NEGATIVE_INFINITE" : "SIGN_POSITIVE_INFINITE");
+        if (isZero()) return getMetaClass().getConstant(zeroSign < 0 ? "SIGN_NEGATIVE_ZERO" : "SIGN_POSITIVE_ZERO");
 
-        if (isInfinity()) {
-            if (infinitySign < 0) {
-                return getMetaClass().getConstant("SIGN_NEGATIVE_INFINITE");
-            } else {
-                return getMetaClass().getConstant("SIGN_POSITIVE_INFINITE");
-            }
-        }
-
-        if (isZero()) {
-            if (zeroSign < 0) {
-                return getMetaClass().getConstant("SIGN_NEGATIVE_ZERO");
-            } else {
-                return getMetaClass().getConstant("SIGN_POSITIVE_ZERO");
-            }
-        }
-
-        if (value.signum() < 0) {
-            return getMetaClass().getConstant("SIGN_NEGATIVE_FINITE");
-        } else {
-            return getMetaClass().getConstant("SIGN_POSITIVE_FINITE");
-        }
+        return getMetaClass().getConstant(value.signum() < 0 ? "SIGN_NEGATIVE_FINITE" : "SIGN_POSITIVE_FINITE");
     }
 
     @JRubyMethod(name = "split")
@@ -1764,6 +1791,17 @@ public class RubyBigDecimal extends RubyNumeric {
         }
     }
 
+    @JRubyMethod(name = "to_r")
+    public IRubyObject to_r(ThreadContext context) {
+        checkFloatDomain();
+
+        int scale = value.scale();
+        BigInteger numerator = value.scaleByPowerOfTen(scale).toBigInteger();
+        BigInteger denominator = BigInteger.valueOf((long)Math.pow(10, scale));
+
+        return RubyRational.newInstance(context, context.runtime.getRational(), RubyBignum.newBignum(context.runtime, numerator), RubyBignum.newBignum(context.runtime, denominator));
+    }
+
     @JRubyMethod(name = {"to_i", "to_int"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject to_int19() {
         checkFloatDomain();
