@@ -36,6 +36,8 @@
 package org.jruby.parser;
 
 import java.math.BigInteger;
+import java.nio.charset.Charset;
+import java.nio.charset.UnsupportedCharsetException;
 import org.jcodings.Encoding;
 import org.jruby.CompatVersion;
 import org.jruby.RubyBignum;
@@ -234,6 +236,12 @@ public class ParserSupport {
                 "identifier " + (String) lhs.getValue() + " is not valid", lhs.getValue());
     }
 
+    // We know it has to be tLABEL or tIDENTIFIER so none of the other assignable logic is needed
+    public AssignableNode assignableInCurr(Token name, Node value) {
+        currentScope.addVariableThisScope(name.getValue().toString());
+        return currentScope.assign(lexer.getPosition(), name.getValue().toString(), makeNullNil(value));
+    }
+
     /**
      *  Wraps node with NEWLINE node.
      *
@@ -1144,6 +1152,22 @@ public class ParserSupport {
     public DStrNode createDStrNode(ISourcePosition position) {
         return new DStrNode(position);
     }
+
+    /**
+     * Used by 1.9 parser (so encoding aware)
+     */
+    public Node asSymbol(ISourcePosition position, String value) {
+        // FIXME: tLABEL and identifiers could return ByteList and not String and make String on-demand for method names
+        // or lvars.  This would prevent this re-extraction of bytes from a string with proper charset
+        try {
+            Charset charset = lexer.getEncoding().getCharset();
+            if (charset != null) return new SymbolNode(position, new ByteList(value.getBytes(charset), lexer.getEncoding()));
+        } catch (UnsupportedCharsetException e) {}
+
+        // for non-charsets we are screwed here since bytes will file.encoding and not what we read them as (see above FIXME for
+        // a much more invasive solution.
+        return new SymbolNode(position, new ByteList(value.getBytes(), lexer.getEncoding()));
+    }
     
     public Node asSymbol(ISourcePosition position, Node value) {
         if (value instanceof StrNode) return new SymbolNode(position, ((StrNode) value).getValue());
