@@ -17,7 +17,7 @@
  * Copyright (C) 2004 Charles O Nutter <headius@headius.com>
  * Copyright (C) 2004 Stefan Matthias Aust <sma@3plus4.de>
  * Copyright (C) 2006 Ola Bini <ola.bini@ki.se>
- * 
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -76,7 +76,7 @@ public class UnmarshalStream extends InputStream {
     public UnmarshalStream(Ruby runtime, InputStream in, IRubyObject proc, boolean taint) throws IOException {
         this(runtime, in, proc, taint, false);
     }
-    
+
     public UnmarshalStream(Ruby runtime, InputStream in, IRubyObject proc, boolean taint, boolean untrust) throws IOException {
         this.runtime = runtime;
         this.cache = new UnmarshalCache(runtime);
@@ -108,31 +108,31 @@ public class UnmarshalStream extends InputStream {
     }
 
     // introduced for keeping ivar read state recursively.
-    private class MarshalState {
+    private static class MarshalState {
         private boolean ivarWaiting;
-        
+
         MarshalState(boolean ivarWaiting) {
             this.ivarWaiting = ivarWaiting;
         }
-        
+
         boolean isIvarWaiting() {
             return ivarWaiting;
         }
-        
+
         void setIvarWaiting(boolean ivarWaiting) {
             this.ivarWaiting = ivarWaiting;
         }
     }
-    
+
     // r_object0
-    public IRubyObject unmarshalObject(MarshalState state) throws IOException {
+    private IRubyObject unmarshalObject(MarshalState state) throws IOException {
         return unmarshalObject(state, true);
     }
 
     // r_object0
-    public IRubyObject unmarshalObject(MarshalState state, boolean callProc) throws IOException {
-        int type = readUnsignedByte();
-        IRubyObject result = null;
+    private IRubyObject unmarshalObject(MarshalState state, boolean callProc) throws IOException {
+        final int type = readUnsignedByte();
+        final IRubyObject result;
         if (cache.isLinkType(type)) {
             result = cache.readLink(this, type);
             if (callProc && runtime.is1_9()) return doCallProcForLink(result, type);
@@ -153,15 +153,29 @@ public class UnmarshalStream extends InputStream {
     }
 
     public static RubyModule getModuleFromPath(Ruby runtime, String path) {
-        RubyModule value = runtime.getClassFromPath(path);
-        if (!value.isModule()) throw runtime.newArgumentError(path + " does not refer module");
+        final RubyModule value = getConstantFromPath(runtime, path);
+        if ( ! value.isModule() ) throw runtime.newArgumentError(path + " does not refer module");
         return value;
     }
 
     public static RubyClass getClassFromPath(Ruby runtime, String path) {
-        RubyModule value = runtime.getClassFromPath(path);
-        if (!value.isClass()) throw runtime.newArgumentError(path + " does not refer class");
-        return (RubyClass)value;
+        final RubyModule value = getConstantFromPath(runtime, path);
+        if ( ! value.isClass() ) throw runtime.newArgumentError(path + " does not refer class");
+        return (RubyClass) value;
+    }
+
+    private static RubyModule getConstantFromPath(Ruby runtime, String path) {
+        final RubyModule value;
+        try {
+            value = runtime.getClassFromPath(path);
+        }
+        catch (RaiseException e) {
+            if ( runtime.getModule("NameError").isInstance(e.getException()) ) {
+                throw runtime.newArgumentError("undefined class/module " + path);
+            }
+            throw e;
+        }
+        return value;
     }
 
     private IRubyObject doCallProcForLink(IRubyObject result, int type) {
@@ -232,20 +246,12 @@ public class UnmarshalStream extends InputStream {
                 break;
             case 'e':
                 RubySymbol moduleName = (RubySymbol) unmarshalObject();
-                RubyModule tp = null;
-                try {
-                    tp = runtime.getClassFromPath(moduleName.asJavaString());
-                } catch (RaiseException e) {
-                    if (runtime.getModule("NameError").isInstance(e.getException())) {
-                        throw runtime.newArgumentError("undefined class/module " + moduleName.asJavaString());
-                    } 
-                    throw e;
-                }
+                final RubyModule tp = getModuleFromPath(runtime, moduleName.asJavaString());
 
                 rubyObj = unmarshalObject();
 
                 tp.extend_object(rubyObj);
-                tp.callMethod(runtime.getCurrentContext(),"extended", rubyObj);
+                tp.callMethod(runtime.getCurrentContext(), "extended", rubyObj);
                 break;
             case 'l' :
                 rubyObj = RubyBignum.unmarshalFrom(this);
@@ -277,7 +283,7 @@ public class UnmarshalStream extends InputStream {
             // call the proc, but not for symbols
             doCallProcForObj(rubyObj);
         }
-        
+
         return rubyObj;
     }
 
@@ -348,18 +354,7 @@ public class UnmarshalStream extends InputStream {
     private IRubyObject defaultObjectUnmarshal() throws IOException {
         RubySymbol className = (RubySymbol) unmarshalObject(false);
 
-        RubyClass type = null;
-        try {
-            type = getClassFromPath(runtime, className.toString());
-        } catch (RaiseException e) {
-            if (runtime.getModule("NameError").isInstance(e.getException())) {
-                throw runtime.newArgumentError("undefined class/module " + className.asJavaString());
-            }
-
-            throw e;
-        }
-
-        assert type != null : "type shouldn't be null.";
+        RubyClass type = getClassFromPath(runtime, className.toString());
 
         IRubyObject result = (IRubyObject)type.unmarshal(this);
 
@@ -370,17 +365,17 @@ public class UnmarshalStream extends InputStream {
         int count = unmarshalInt();
 
         RubyClass cls = object.getMetaClass().getRealClass();
-        
+
         for (int i = 0; i < count; i++) {
-            
+
             IRubyObject key = unmarshalObject(false);
-            
+
             if (runtime.is1_9() && object instanceof EncodingCapable) {
-                
+
                 EncodingCapable strObj = (EncodingCapable)object;
 
                 if (key.asJavaString().equals(MarshalStream.SYMBOL_ENCODING_SPECIAL)) {
-                    
+
                     // special case for USASCII and UTF8
                     if (unmarshalObject().isTrue()) {
                         strObj.setEncoding(UTF8Encoding.INSTANCE);
@@ -388,9 +383,9 @@ public class UnmarshalStream extends InputStream {
                         strObj.setEncoding(USASCIIEncoding.INSTANCE);
                     }
                     continue;
-                    
+
                 } else if (key.asJavaString().equals("encoding")) {
-                    
+
                     IRubyObject encodingNameObj = unmarshalObject(false);
                     String encodingNameStr = encodingNameObj.asJavaString();
                     ByteList encodingName = new ByteList(ByteList.plain(encodingNameStr));
@@ -402,10 +397,10 @@ public class UnmarshalStream extends InputStream {
                     Encoding encoding = entry.getEncoding();
                     strObj.setEncoding(encoding);
                     continue;
-                    
+
                 } // else fall through as normal ivar
             }
-            
+
             String name = key.asJavaString();
             IRubyObject value = unmarshalObject();
 
@@ -416,7 +411,7 @@ public class UnmarshalStream extends InputStream {
     private IRubyObject uclassUnmarshall() throws IOException {
         RubySymbol className = (RubySymbol)unmarshalObject(false);
 
-        RubyClass type = (RubyClass)runtime.getClassFromPath(className.asJavaString());
+        RubyClass type = getClassFromPath(runtime, className.asJavaString());
 
         // singleton, raise error
         if (type.isSingleton()) throw runtime.newTypeError("singleton can't be loaded");
@@ -442,7 +437,7 @@ public class UnmarshalStream extends InputStream {
         String className = unmarshalObject(false).asJavaString();
         ByteList marshaled = unmarshalString();
         RubyClass classInstance = findClass(className);
-        RubyString data = RubyString.newString(getRuntime(), marshaled);
+        RubyString data = RubyString.newString(runtime, marshaled);
         if (state.isIvarWaiting()) {
             defaultVariablesUnmarshal(data);
             state.setIvarWaiting(false);
@@ -462,19 +457,7 @@ public class UnmarshalStream extends InputStream {
     }
 
     private RubyClass findClass(String className) {
-        RubyModule classInstance;
-        try {
-            classInstance = runtime.getClassFromPath(className);
-        } catch (RaiseException e) {
-            if (runtime.getModule("NameError").isInstance(e.getException())) {
-                throw runtime.newArgumentError("undefined class/module " + className);
-            } 
-            throw e;
-        }
-        if (! (classInstance instanceof RubyClass)) {
-            throw runtime.newArgumentError(className + " does not refer class"); // sic
-        }
-        return (RubyClass) classInstance;
+        return getClassFromPath(runtime, className);
     }
 
     public int read() throws IOException {
