@@ -60,7 +60,7 @@ import org.jruby.util.log.LoggerFactory;
 /**
  * This file implements a seekable IO file.
  */
-public class ChannelStream implements Stream, Finalizable {
+public class ChannelStream implements Stream, Finalizable, NonblockWritingStream {
 
     private static final Logger LOG = LoggerFactory.getLogger("ChannelStream");
 
@@ -146,7 +146,7 @@ public class ChannelStream implements Stream, Finalizable {
     public ModeFlags getModes() {
     	return modes;
     }
-    
+
     public void setModes(ModeFlags modes) {
         this.modes = modes;
     }
@@ -162,7 +162,7 @@ public class ChannelStream implements Stream, Finalizable {
     public void setBinmode() {
         // No-op here, no binmode handling needed.
     }
-    
+
     public boolean isBinmode() {
         return false;
     }
@@ -187,15 +187,15 @@ public class ChannelStream implements Stream, Finalizable {
         }
     }
 
-    public boolean readDataBuffered() {
-        return reading && (hasUngotChars() || buffer.hasRemaining());
+    public final boolean readDataBuffered() {
+        return hasBufferedInputBytes();
     }
 
     private boolean hasUngotChars() {
         return ungotChars.length() > 0;
     }
 
-    public boolean writeDataBuffered() {
+    public final boolean writeDataBuffered() {
         return !reading && buffer.position() > 0;
     }
 
@@ -295,7 +295,7 @@ public class ChannelStream implements Stream, Finalizable {
 
         int totalRead = 0;
         boolean found = false;
-        
+
         if (hasUngotChars()) {
             for(int i = 0; i < ungotChars.length(); i++){
                 byte ungotc = (byte) ungotChars.get(i);
@@ -305,7 +305,7 @@ public class ChannelStream implements Stream, Finalizable {
             }
             clearUngotChars();
         }
-        
+
         while (!found) {
             final byte[] bytes = buffer.array();
             final int begin = buffer.arrayOffset() + buffer.position();
@@ -339,7 +339,7 @@ public class ChannelStream implements Stream, Finalizable {
 
         int totalRead = 0;
         boolean found = false;
-        
+
         if (hasUngotChars()) {
             for(int i = 0; i < ungotChars.length(); i++){
                 byte ungotc = (byte) ungotChars.get(i);
@@ -350,7 +350,7 @@ public class ChannelStream implements Stream, Finalizable {
             }
             clearUngotChars();
         }
-        
+
         while (!found) {
             final byte[] bytes = buffer.array();
             final int begin = buffer.arrayOffset() + buffer.position();
@@ -380,7 +380,7 @@ public class ChannelStream implements Stream, Finalizable {
     }
 
     /**
-     * 
+     *
      */
     private void clearUngotChars() {
         if(ungotChars.length() > 0) {
@@ -481,7 +481,7 @@ public class ChannelStream implements Stream, Finalizable {
      */
     private final int copyBufferedBytes(ByteBuffer dst) {
         final int bytesToCopy = dst.remaining();
-        
+
         if (hasUngotChars() && dst.hasRemaining()) {
             for(int i = 0; i < ungotChars.length(); i++){
                 byte ungotc = (byte) ungotChars.get(i);
@@ -520,7 +520,7 @@ public class ChannelStream implements Stream, Finalizable {
      */
     private final int copyBufferedBytes(byte[] dst, int off, int len) {
         int bytesCopied = 0;
-        
+
         if (hasUngotChars() && len > 0) {
             for(int i = 0; i < ungotChars.length(); i++){
                 byte ungotc = (byte) ungotChars.get(i);
@@ -548,7 +548,7 @@ public class ChannelStream implements Stream, Finalizable {
         int bytesCopied = 0;
 
         dst.ensure(Math.min(len, bufferedInputBytesRemaining()));
-        
+
         if (hasUngotChars() && hasUngotChars()) {
             for(int i = 0; i < ungotChars.length(); i++){
                 byte ungotc = (byte) ungotChars.get(i);
@@ -824,6 +824,7 @@ public class ChannelStream implements Stream, Finalizable {
 
     public synchronized void sync() throws IOException, BadDescriptorException {
         flushWrite();
+        descriptor.fsync();
     }
 
     /**
@@ -1195,7 +1196,7 @@ public class ChannelStream implements Stream, Finalizable {
     @Override
     public void finalize() throws Throwable {
         super.finalize();
-        
+
         if (closedExplicitly) return;
 
         if (DEBUG) {
