@@ -1,8 +1,8 @@
 # = uri/common.rb
 #
 # Author:: Akira Yamada <akira@ruby-lang.org>
-# Revision:: $Id: common.rb 24773 2009-09-06 18:24:53Z naruse $
-# License::
+# Revision:: $Id$
+# License:: 
 #   You can redistribute it and/or modify it under the same term as Ruby.
 #
 
@@ -34,10 +34,20 @@ module URI
       UNRESERVED = "-_.!~*'()#{ALNUM}"
       # reserved      = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" |
       #                 "$" | ","
-      # reserved      = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" |
+      # reserved      = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" | 
       #                 "$" | "," | "[" | "]" (RFC 2732)
       RESERVED = ";/?:@&=+$,\\[\\]"
 
+      # uric          = reserved | unreserved | escaped
+      URIC = "(?:[#{UNRESERVED}#{RESERVED}]|#{ESCAPED})"
+      # uric_no_slash = unreserved | escaped | ";" | "?" | ":" | "@" |
+      #                 "&" | "=" | "+" | "$" | ","
+      URIC_NO_SLASH = "(?:[#{UNRESERVED};?:@&=+$,]|#{ESCAPED})"
+      # query         = *uric
+      QUERY = "#{URIC}*"
+      # fragment      = *uric
+      FRAGMENT = "#{URIC}*"
+
       # domainlabel   = alphanum | alphanum *( alphanum | "-" ) alphanum
       DOMLABEL = "(?:[#{ALNUM}](?:[-#{ALNUM}]*[#{ALNUM}])?)"
       # toplabel      = alpha | alpha *( alphanum | "-" ) alphanum
@@ -45,226 +55,6 @@ module URI
       # hostname      = *( domainlabel "." ) toplabel [ "." ]
       HOSTNAME = "(?:#{DOMLABEL}\\.)*#{TOPLABEL}\\.?"
 
-      # :startdoc:
-    end # PATTERN
-
-    # :startdoc:
-  end # REGEXP
-
-  class Parser
-    include REGEXP
-
-    #
-    # == Synopsis
-    #
-    #   URI::Parser.new([opts])
-    #
-    # == Args
-    #
-    # The constructor accepts a hash as options for parser.
-    # Keys of options are pattern names of URI components
-    # and values of options are pattern strings.
-    # The constructor generetes set of regexps for parsing URIs.
-    #
-    # You can use the following keys:
-    #
-    #   * <tt>:ESCAPED</tt> (URI::PATTERN::ESCAPED in default)
-    #   * <tt>:UNRESERVED</tt> (URI::PATTERN::UNRESERVED in default)
-    #   * <tt>:DOMLABEL</tt> (URI::PATTERN::DOMLABEL in default)
-    #   * <tt>:TOPLABEL</tt> (URI::PATTERN::TOPLABEL in default)
-    #   * <tt>:HOSTNAME</tt> (URI::PATTERN::HOSTNAME in default)
-    #
-    # == Examples
-    #
-    #   p = URI::Parser.new(:ESCPAED => "(?:%[a-fA-F0-9]{2}|%u[a-fA-F0-9]{4})"
-    #   u = p.parse("http://example.jp/%uABCD") #=> #<URI::HTTP:0xb78cf4f8 URL:http://example.jp/%uABCD>
-    #   URI.parse(u.to_s) #=> raises URI::InvalidURIError
-    #
-    #   s = "http://examle.com/ABCD"
-    #   u1 = p.parse(s) #=> #<URI::HTTP:0xb78c3220 URL:http://example.com/ABCD>
-    #   u2 = URI.parse(s) #=> #<URI::HTTP:0xb78b6d54 URL:http://example.com/ABCD>
-    #   u1 == u2 #=> true
-    #   u1.eql?(u2) #=> false
-    #
-    def initialize(opts = {})
-      @pattern = initialize_pattern(opts)
-      @pattern.each_value {|v| v.freeze}
-      @pattern.freeze
-
-      @regexp = initialize_regexp(@pattern)
-      @regexp.each_value {|v| v.freeze}
-      @regexp.freeze
-    end
-    attr_reader :pattern, :regexp
-
-    def split(uri)
-      case uri
-      when ''
-	# null uri
-
-      when @regexp[:ABS_URI]
-	scheme, opaque, userinfo, host, port,
-	  registry, path, query, fragment = $~[1..-1]
-
-	# URI-reference = [ absoluteURI | relativeURI ] [ "#" fragment ]
-
-	# absoluteURI   = scheme ":" ( hier_part | opaque_part )
-	# hier_part     = ( net_path | abs_path ) [ "?" query ]
-	# opaque_part   = uric_no_slash *uric
-
-	# abs_path      = "/"  path_segments
-	# net_path      = "//" authority [ abs_path ]
-
-	# authority     = server | reg_name
-	# server        = [ [ userinfo "@" ] hostport ]
-
-	if !scheme
-	  raise InvalidURIError,
-	    "bad URI(absolute but no scheme): #{uri}"
-	end
-	if !opaque && (!path && (!host && !registry))
-	  raise InvalidURIError,
-	    "bad URI(absolute but no path): #{uri}"
-	end
-
-      when @regexp[:REL_URI]
-	scheme = nil
-	opaque = nil
-
-	userinfo, host, port, registry,
-	  rel_segment, abs_path, query, fragment = $~[1..-1]
-	if rel_segment && abs_path
-	  path = rel_segment + abs_path
-	elsif rel_segment
-	  path = rel_segment
-	elsif abs_path
-	  path = abs_path
-	end
-
-	# URI-reference = [ absoluteURI | relativeURI ] [ "#" fragment ]
-
-	# relativeURI   = ( net_path | abs_path | rel_path ) [ "?" query ]
-
-	# net_path      = "//" authority [ abs_path ]
-	# abs_path      = "/"  path_segments
-	# rel_path      = rel_segment [ abs_path ]
-
-	# authority     = server | reg_name
-	# server        = [ [ userinfo "@" ] hostport ]
-
-      else
-	raise InvalidURIError, "bad URI(is not URI?): #{uri}"
-      end
-
-      path = '' if !path && !opaque # (see RFC2396 Section 5.2)
-      ret = [
-	scheme,
-	userinfo, host, port,         # X
-	registry,                     # X
-	path,                         # Y
-	opaque,                       # Y
-	query,
-	fragment
-      ]
-      return ret
-    end
-
-    def parse(uri)
-      scheme, userinfo, host, port,
-       	registry, path, opaque, query, fragment = self.split(uri)
-
-      if scheme && URI.scheme_list.include?(scheme.upcase)
-	URI.scheme_list[scheme.upcase].new(scheme, userinfo, host, port,
-                                           registry, path, opaque, query,
-                                           fragment, self)
-      else
-	Generic.new(scheme, userinfo, host, port,
-	   	    registry, path, opaque, query,
-	    	    fragment, self)
-      end
-    end
-
-    def join(*str)
-      u = self.parse(str[0])
-      str[1 .. -1].each do |x|
-	u = u.merge(x)
-      end
-      u
-    end
-
-    def extract(str, schemes = nil, &block)
-      if block_given?
-       	str.scan(make_regexp(schemes)) { yield $& }
-	nil
-      else
-	result = []
-	str.scan(make_regexp(schemes)) { result.push $& }
-	result
-      end
-    end
-
-    def make_regexp(schemes = nil)
-      unless schemes
-       	@regexp[:ABS_URI_REF]
-      else
-	/(?=#{Regexp.union(*schemes)}:)#{@pattern[:X_ABS_URI]}/x
-      end
-    end
-
-    def escape(str, unsafe = @regexp[:UNSAFE])
-      unless unsafe.kind_of?(Regexp)
-        # perhaps unsafe is String object
-        unsafe = Regexp.new("[#{Regexp.quote(unsafe)}]", false)
-      end
-      str.gsub(unsafe) do
-        us = $&
-        tmp = ''
-        us.each_byte do |uc|
-          tmp << sprintf('%%%02X', uc)
-        end
-        tmp
-      end.force_encoding(Encoding::US_ASCII)
-    end
-
-    def unescape(str, escaped = @regexp[:ESCAPED])
-      str.gsub(escaped) { [$&[1, 2].hex].pack('C') }.force_encoding(str.encoding)
-    end
-
-    @@to_s = Kernel.instance_method(:to_s)
-    def inspect
-      @@to_s.bind(self).call
-    end
-
-    private
-
-    def initialize_pattern(opts = {})
-      ret = {}
-      ret[:ESCAPED] = escaped = (opts.delete(:ESCAPED) || PATTERN::ESCAPED)
-      ret[:UNRESERVED] = unreserved = opts.delete(:UNRESERVED) || PATTERN::UNRESERVED
-      ret[:RESERVED] = reserved = opts.delete(:RESERVED) || PATTERN::RESERVED
-      ret[:DOMLABEL] = domlabel = opts.delete(:DOMLABEL) || PATTERN::DOMLABEL
-      ret[:TOPLABEL] = toplabel = opts.delete(:TOPLABEL) || PATTERN::TOPLABEL
-      ret[:HOSTNAME] = hostname = opts.delete(:HOSTNAME)
-
-      # RFC 2396 (URI Generic Syntax)
-      # RFC 2732 (IPv6 Literal Addresses in URL's)
-      # RFC 2373 (IPv6 Addressing Architecture)
-
-      # uric          = reserved | unreserved | escaped
-      ret[:URIC] = uric = "(?:[#{unreserved}#{reserved}]|#{escaped})"
-      # uric_no_slash = unreserved | escaped | ";" | "?" | ":" | "@" |
-      #                 "&" | "=" | "+" | "$" | ","
-      ret[:URIC_NO_SLASH] = uric_no_slash = "(?:[#{unreserved};?:@&=+$,]|#{escaped})"
-      # query         = *uric
-      ret[:QUERY] = query = "#{uric}*"
-      # fragment      = *uric
-      ret[:FRAGMENT] = fragment = "#{uric}*"
-
-      # hostname      = *( domainlabel "." ) toplabel [ "." ]
-      unless hostname
-	ret[:HOSTNAME] = hostname = "(?:#{domlabel}\\.)*#{toplabel}\\.?"
-      end
-
       # RFC 2373, APPENDIX B:
       # IPv6address = hexpart [ ":" IPv4address ]
       # IPv4address   = 1*3DIGIT "." 1*3DIGIT "." 1*3DIGIT "." 1*3DIGIT
@@ -276,164 +66,153 @@ module URI
       # allowed too.  Here is a replacement.
       #
       # IPv4address = 1*3DIGIT "." 1*3DIGIT "." 1*3DIGIT "." 1*3DIGIT
-      ret[:IPV4ADDR] = ipv4addr = "\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}"
+      IPV4ADDR = "\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}"
       # hex4     = 1*4HEXDIG
-      hex4 = "[#{PATTERN::HEX}]{1,4}"
+      HEX4 = "[#{HEX}]{1,4}"
       # lastpart = hex4 | IPv4address
-      lastpart = "(?:#{hex4}|#{ipv4addr})"
+      LASTPART = "(?:#{HEX4}|#{IPV4ADDR})"
       # hexseq1  = *( hex4 ":" ) hex4
-      hexseq1 = "(?:#{hex4}:)*#{hex4}"
+      HEXSEQ1 = "(?:#{HEX4}:)*#{HEX4}"
       # hexseq2  = *( hex4 ":" ) lastpart
-      hexseq2 = "(?:#{hex4}:)*#{lastpart}"
+      HEXSEQ2 = "(?:#{HEX4}:)*#{LASTPART}"
       # IPv6address = hexseq2 | [ hexseq1 ] "::" [ hexseq2 ]
-      ret[:IPV6ADDR] = ipv6addr = "(?:#{hexseq2}|(?:#{hexseq1})?::(?:#{hexseq2})?)"
+      IPV6ADDR = "(?:#{HEXSEQ2}|(?:#{HEXSEQ1})?::(?:#{HEXSEQ2})?)"
 
       # IPv6prefix  = ( hexseq1 | [ hexseq1 ] "::" [ hexseq1 ] ) "/" 1*2DIGIT
       # unused
 
       # ipv6reference = "[" IPv6address "]" (RFC 2732)
-      ret[:IPV6REF] = ipv6ref = "\\[#{ipv6addr}\\]"
+      IPV6REF = "\\[#{IPV6ADDR}\\]"
 
       # host          = hostname | IPv4address
       # host          = hostname | IPv4address | IPv6reference (RFC 2732)
-      ret[:HOST] = host = "(?:#{hostname}|#{ipv4addr}|#{ipv6ref})"
+      HOST = "(?:#{HOSTNAME}|#{IPV4ADDR}|#{IPV6REF})"
       # port          = *digit
-      port = '\d*'
+      PORT = '\d*'
       # hostport      = host [ ":" port ]
-      ret[:HOSTPORT] = hostport = "#{host}(?::#{port})?"
+      HOSTPORT = "#{HOST}(?::#{PORT})?"
 
       # userinfo      = *( unreserved | escaped |
       #                    ";" | ":" | "&" | "=" | "+" | "$" | "," )
-      ret[:USERINFO] = userinfo = "(?:[#{unreserved};:&=+$,]|#{escaped})*"
+      USERINFO = "(?:[#{UNRESERVED};:&=+$,]|#{ESCAPED})*"
 
       # pchar         = unreserved | escaped |
       #                 ":" | "@" | "&" | "=" | "+" | "$" | ","
-      pchar = "(?:[#{unreserved}:@&=+$,]|#{escaped})"
+      PCHAR = "(?:[#{UNRESERVED}:@&=+$,]|#{ESCAPED})"
       # param         = *pchar
-      param = "#{pchar}*"
+      PARAM = "#{PCHAR}*"
       # segment       = *pchar *( ";" param )
-      segment = "#{pchar}*(?:;#{param})*"
+      SEGMENT = "#{PCHAR}*(?:;#{PARAM})*"
       # path_segments = segment *( "/" segment )
-      ret[:PATH_SEGMENTS] = path_segments = "#{segment}(?:/#{segment})*"
+      PATH_SEGMENTS = "#{SEGMENT}(?:/#{SEGMENT})*"
 
       # server        = [ [ userinfo "@" ] hostport ]
-      server = "(?:#{userinfo}@)?#{hostport}"
+      SERVER = "(?:#{USERINFO}@)?#{HOSTPORT}"
       # reg_name      = 1*( unreserved | escaped | "$" | "," |
       #                     ";" | ":" | "@" | "&" | "=" | "+" )
-      ret[:REG_NAME] = reg_name = "(?:[#{unreserved}$,;:@&=+]|#{escaped})+"
+      REG_NAME = "(?:[#{UNRESERVED}$,;:@&=+]|#{ESCAPED})+"
       # authority     = server | reg_name
-      authority = "(?:#{server}|#{reg_name})"
+      AUTHORITY = "(?:#{SERVER}|#{REG_NAME})"
 
       # rel_segment   = 1*( unreserved | escaped |
       #                     ";" | "@" | "&" | "=" | "+" | "$" | "," )
-      ret[:REL_SEGMENT] = rel_segment = "(?:[#{unreserved};@&=+$,]|#{escaped})+"
+      REL_SEGMENT = "(?:[#{UNRESERVED};@&=+$,]|#{ESCAPED})+"
 
       # scheme        = alpha *( alpha | digit | "+" | "-" | "." )
-      ret[:SCHEME] = scheme = "[#{PATTERN::ALPHA}][-+.#{PATTERN::ALPHA}\\d]*"
+      SCHEME = "[#{ALPHA}][-+.#{ALPHA}\\d]*"
 
       # abs_path      = "/"  path_segments
-      ret[:ABS_PATH] = abs_path = "/#{path_segments}"
+      ABS_PATH = "/#{PATH_SEGMENTS}"
       # rel_path      = rel_segment [ abs_path ]
-      ret[:REL_PATH] = rel_path = "#{rel_segment}(?:#{abs_path})?"
+      REL_PATH = "#{REL_SEGMENT}(?:#{ABS_PATH})?"
       # net_path      = "//" authority [ abs_path ]
-      ret[:NET_PATH] = net_path = "//#{authority}(?:#{abs_path})?"
+      NET_PATH   = "//#{AUTHORITY}(?:#{ABS_PATH})?"
 
       # hier_part     = ( net_path | abs_path ) [ "?" query ]
-      ret[:HIER_PART] = hier_part = "(?:#{net_path}|#{abs_path})(?:\\?(?:#{query}))?"
+      HIER_PART   = "(?:#{NET_PATH}|#{ABS_PATH})(?:\\?(?:#{QUERY}))?"
       # opaque_part   = uric_no_slash *uric
-      ret[:OPAQUE_PART] = opaque_part = "#{uric_no_slash}#{uric}*"
+      OPAQUE_PART = "#{URIC_NO_SLASH}#{URIC}*"
 
       # absoluteURI   = scheme ":" ( hier_part | opaque_part )
-      ret[:ABS_URI] = abs_uri = "#{scheme}:(?:#{hier_part}|#{opaque_part})"
+      ABS_URI   = "#{SCHEME}:(?:#{HIER_PART}|#{OPAQUE_PART})"
       # relativeURI   = ( net_path | abs_path | rel_path ) [ "?" query ]
-      ret[:REL_URI] = rel_uri = "(?:#{net_path}|#{abs_path}|#{rel_path})(?:\\?#{query})?"
+      REL_URI = "(?:#{NET_PATH}|#{ABS_PATH}|#{REL_PATH})(?:\\?#{QUERY})?"
 
       # URI-reference = [ absoluteURI | relativeURI ] [ "#" fragment ]
-      ret[:URI_REF] = uri_ref = "(?:#{abs_uri}|#{rel_uri})?(?:##{fragment})?"
+      URI_REF = "(?:#{ABS_URI}|#{REL_URI})?(?:##{FRAGMENT})?"
 
-      ret[:X_ABS_URI] = "
-        (#{scheme}):                           (?# 1: scheme)
+      # XXX:
+      X_ABS_URI = "
+        (#{PATTERN::SCHEME}):                     (?# 1: scheme)
         (?:
-           (#{opaque_part})                    (?# 2: opaque)
+           (#{PATTERN::OPAQUE_PART})              (?# 2: opaque)
         |
            (?:(?:
              //(?:
-                 (?:(?:(#{userinfo})@)?        (?# 3: userinfo)
-                   (?:(#{host})(?::(\\d*))?))? (?# 4: host, 5: port)
+                 (?:(?:(#{PATTERN::USERINFO})@)?  (?# 3: userinfo)
+                   (?:(#{PATTERN::HOST})(?::(\\d*))?))?(?# 4: host, 5: port)
                |
-                 (#{reg_name})                 (?# 6: registry)
+                 (#{PATTERN::REG_NAME})           (?# 6: registry)
                )
              |
-             (?!//))                           (?# XXX: '//' is the mark for hostport)
-             (#{abs_path})?                    (?# 7: path)
-           )(?:\\?(#{query}))?                 (?# 8: query)
+             (?!//))                              (?# XXX: '//' is the mark for hostport)
+             (#{PATTERN::ABS_PATH})?              (?# 7: path)
+           )(?:\\?(#{PATTERN::QUERY}))?           (?# 8: query)
         )
-        (?:\\#(#{fragment}))?                  (?# 9: fragment)
+        (?:\\#(#{PATTERN::FRAGMENT}))?            (?# 9: fragment)
       "
-
-      ret[:X_REL_URI] = "
+      X_REL_URI = "
         (?:
           (?:
             //
             (?:
-              (?:(#{userinfo})@)?       (?# 1: userinfo)
-                (#{host})?(?::(\\d*))?  (?# 2: host, 3: port)
+              (?:(#{PATTERN::USERINFO})@)?       (?# 1: userinfo)
+                (#{PATTERN::HOST})?(?::(\\d*))?  (?# 2: host, 3: port)
             |
-              (#{reg_name})             (?# 4: registry)
+              (#{PATTERN::REG_NAME})             (?# 4: registry)
             )
           )
         |
-          (#{rel_segment})              (?# 5: rel_segment)
+          (#{PATTERN::REL_SEGMENT})              (?# 5: rel_segment)
         )?
-        (#{abs_path})?                  (?# 6: abs_path)
-        (?:\\?(#{query}))?              (?# 7: query)
-        (?:\\#(#{fragment}))?           (?# 8: fragment)
+        (#{PATTERN::ABS_PATH})?                  (?# 6: abs_path)
+        (?:\\?(#{PATTERN::QUERY}))?              (?# 7: query)
+        (?:\\#(#{PATTERN::FRAGMENT}))?           (?# 8: fragment)
       "
+      # :startdoc:
+    end # PATTERN
 
-      ret
-    end
+    # :stopdoc:
 
-    def initialize_regexp(pattern)
-      ret = {}
+    # for URI::split
+    ABS_URI = Regexp.new('^' + PATTERN::X_ABS_URI + '$', #'
+                         Regexp::EXTENDED, 'N').freeze
+    REL_URI = Regexp.new('^' + PATTERN::X_REL_URI + '$', #'
+                         Regexp::EXTENDED, 'N').freeze
 
-      # for URI::split
-      ret[:ABS_URI] = Regexp.new('^' + pattern[:X_ABS_URI] + '$', Regexp::EXTENDED)
-      ret[:REL_URI] = Regexp.new('^' + pattern[:X_REL_URI] + '$', Regexp::EXTENDED)
+    # for URI::extract
+    URI_REF     = Regexp.new(PATTERN::URI_REF, false, 'N').freeze
+    ABS_URI_REF = Regexp.new(PATTERN::X_ABS_URI, Regexp::EXTENDED, 'N').freeze
+    REL_URI_REF = Regexp.new(PATTERN::X_REL_URI, Regexp::EXTENDED, 'N').freeze
 
-      # for URI::extract
-      ret[:URI_REF]     = Regexp.new(pattern[:URI_REF])
-      ret[:ABS_URI_REF] = Regexp.new(pattern[:X_ABS_URI], Regexp::EXTENDED)
-      ret[:REL_URI_REF] = Regexp.new(pattern[:X_REL_URI], Regexp::EXTENDED)
+    # for URI::escape/unescape
+    ESCAPED = Regexp.new(PATTERN::ESCAPED, false, 'N').freeze
+    UNSAFE  = Regexp.new("[^#{PATTERN::UNRESERVED}#{PATTERN::RESERVED}]",
+                         false, 'N').freeze
 
-      # for URI::escape/unescape
-      ret[:ESCAPED] = Regexp.new(pattern[:ESCAPED])
-      ret[:UNSAFE]  = Regexp.new("[^#{pattern[:UNRESERVED]}#{pattern[:RESERVED]}]")
-
-      # for Generic#initialize
-      ret[:SCHEME]   = Regexp.new("^#{pattern[:SCHEME]}$")
-      ret[:USERINFO] = Regexp.new("^#{pattern[:USERINFO]}$")
-      ret[:HOST]     = Regexp.new("^#{pattern[:HOST]}$")
-      ret[:PORT]     = Regexp.new("^#{pattern[:PORT]}$")
-      ret[:OPAQUE]   = Regexp.new("^#{pattern[:OPAQUE_PART]}$")
-      ret[:REGISTRY] = Regexp.new("^#{pattern[:REG_NAME]}$")
-      ret[:ABS_PATH] = Regexp.new("^#{pattern[:ABS_PATH]}$")
-      ret[:REL_PATH] = Regexp.new("^#{pattern[:REL_PATH]}$")
-      ret[:QUERY]    = Regexp.new("^#{pattern[:QUERY]}$")
-      ret[:FRAGMENT] = Regexp.new("^#{pattern[:FRAGMENT]}$")
-
-      ret
-    end
-  end # class Parser
-
-  DEFAULT_PARSER = Parser.new
-  DEFAULT_PARSER.pattern.each_pair do |sym, str|
-    unless REGEXP::PATTERN.const_defined?(sym)
-      REGEXP::PATTERN.const_set(sym, str)
-    end
-  end
-  DEFAULT_PARSER.regexp.each_pair do |sym, str|
-    const_set(sym, str)
-  end
+    # for Generic#initialize
+    SCHEME   = Regexp.new("^#{PATTERN::SCHEME}$", false, 'N').freeze #"
+    USERINFO = Regexp.new("^#{PATTERN::USERINFO}$", false, 'N').freeze #"
+    HOST     = Regexp.new("^#{PATTERN::HOST}$", false, 'N').freeze #"
+    PORT     = Regexp.new("^#{PATTERN::PORT}$", false, 'N').freeze #"
+    OPAQUE   = Regexp.new("^#{PATTERN::OPAQUE_PART}$", false, 'N').freeze #"
+    REGISTRY = Regexp.new("^#{PATTERN::REG_NAME}$", false, 'N').freeze #"
+    ABS_PATH = Regexp.new("^#{PATTERN::ABS_PATH}$", false, 'N').freeze #"
+    REL_PATH = Regexp.new("^#{PATTERN::REL_PATH}$", false, 'N').freeze #"
+    QUERY    = Regexp.new("^#{PATTERN::QUERY}$", false, 'N').freeze #"
+    FRAGMENT = Regexp.new("^#{PATTERN::FRAGMENT}$", false, 'N').freeze #"
+    # :startdoc:
+  end # REGEXP
 
   module Util # :nodoc:
     def make_components_hash(klass, array_hash)
@@ -457,7 +236,7 @@ module URI
           end
         end
       else
-        raise ArgumentError,
+        raise ArgumentError, 
           "expected Array of or Hash of components of #{klass.to_s} (#{klass.component[1..-1].join(', ')})"
       end
       tmp[:scheme] = klass.to_s.sub(/\A.*::/, '').downcase
@@ -468,6 +247,8 @@ module URI
   end
 
   module Escape
+    include REGEXP
+
     #
     # == Synopsis
     #
@@ -500,9 +281,18 @@ module URI
     #   p URI.escape("@?@!", "!?")
     #   # => "@%3F@%21"
     #
-    def escape(*arg)
-      warn "#{caller(1)[0]}: warning: URI.escape is obsolete" if $VERBOSE
-      DEFAULT_PARSER.escape(*arg)
+    def escape(str, unsafe = UNSAFE)
+      unless unsafe.kind_of?(Regexp)
+        # perhaps unsafe is String object
+        unsafe = Regexp.new("[#{Regexp.quote(unsafe)}]", false, 'N')
+      end
+      str.gsub(unsafe) do |us|
+        tmp = ''
+        us.each_byte do |uc|
+          tmp << sprintf('%%%02X', uc)
+        end
+        tmp
+      end
     end
     alias encode escape
     #
@@ -526,21 +316,19 @@ module URI
     #   p URI.unescape(enc_uri)
     #   # => "http://example.com/?a=\t\r"
     #
-    def unescape(*arg)
-      warn "#{caller(1)[0]}: warning: URI.unescape is obsolete" if $VERBOSE
-      DEFAULT_PARSER.unescape(*arg)
+    def unescape(str)
+      str.gsub(ESCAPED) do
+        $&[1,2].hex.chr
+      end
     end
     alias decode unescape
   end
 
-  extend Escape
   include REGEXP
+  extend Escape
 
   @@schemes = {}
-  def self.scheme_list
-    @@schemes
-  end
-
+  
   #
   # Base class for all URI exceptions.
   #
@@ -581,7 +369,7 @@ module URI
   #   * Opaque
   #   * Query
   #   * Fragment
-  #
+  # 
   # == Usage
   #
   #   require 'uri'
@@ -590,7 +378,75 @@ module URI
   #   # => ["http", nil, "www.ruby-lang.org", nil, nil, "/", nil, nil, nil]
   #
   def self.split(uri)
-    DEFAULT_PARSER.split(uri)
+    case uri
+    when ''
+      # null uri
+
+    when ABS_URI
+      scheme, opaque, userinfo, host, port, 
+        registry, path, query, fragment = $~[1..-1]
+
+      # URI-reference = [ absoluteURI | relativeURI ] [ "#" fragment ]
+
+      # absoluteURI   = scheme ":" ( hier_part | opaque_part )
+      # hier_part     = ( net_path | abs_path ) [ "?" query ]
+      # opaque_part   = uric_no_slash *uric
+
+      # abs_path      = "/"  path_segments
+      # net_path      = "//" authority [ abs_path ]
+
+      # authority     = server | reg_name
+      # server        = [ [ userinfo "@" ] hostport ]
+
+      if !scheme
+        raise InvalidURIError, 
+          "bad URI(absolute but no scheme): #{uri}"
+      end
+      if !opaque && (!path && (!host && !registry))
+        raise InvalidURIError,
+          "bad URI(absolute but no path): #{uri}" 
+      end
+
+    when REL_URI
+      scheme = nil
+      opaque = nil
+
+      userinfo, host, port, registry, 
+        rel_segment, abs_path, query, fragment = $~[1..-1]
+      if rel_segment && abs_path
+        path = rel_segment + abs_path
+      elsif rel_segment
+        path = rel_segment
+      elsif abs_path
+        path = abs_path
+      end
+
+      # URI-reference = [ absoluteURI | relativeURI ] [ "#" fragment ]
+
+      # relativeURI   = ( net_path | abs_path | rel_path ) [ "?" query ]
+
+      # net_path      = "//" authority [ abs_path ]
+      # abs_path      = "/"  path_segments
+      # rel_path      = rel_segment [ abs_path ]
+
+      # authority     = server | reg_name
+      # server        = [ [ userinfo "@" ] hostport ]
+
+    else
+      raise InvalidURIError, "bad URI(is not URI?): #{uri}"
+    end
+
+    path = '' if !path && !opaque # (see RFC2396 Section 5.2)
+    ret = [
+      scheme, 
+      userinfo, host, port,         # X
+      registry,                        # X
+      path,                         # Y
+      opaque,                        # Y
+      query,
+      fragment
+    ]
+    return ret
   end
 
   #
@@ -606,7 +462,7 @@ module URI
   # == Description
   #
   # Creates one of the URI's subclasses instance from the string.
-  #
+  #  
   # == Raises
   #
   # URI::InvalidURIError
@@ -619,13 +475,24 @@ module URI
   #   uri = URI.parse("http://www.ruby-lang.org/")
   #   p uri
   #   # => #<URI::HTTP:0x202281be URL:http://www.ruby-lang.org/>
-  #   p uri.scheme
-  #   # => "http"
-  #   p uri.host
-  #   # => "www.ruby-lang.org"
-  #
+  #   p uri.scheme 
+  #   # => "http" 
+  #   p uri.host 
+  #   # => "www.ruby-lang.org" 
+  # 
   def self.parse(uri)
-    DEFAULT_PARSER.parse(uri)
+    scheme, userinfo, host, port, 
+      registry, path, opaque, query, fragment = self.split(uri)
+
+    if scheme && @@schemes.include?(scheme.upcase)
+      @@schemes[scheme.upcase].new(scheme, userinfo, host, port, 
+                                   registry, path, opaque, query, 
+                                   fragment)
+    else
+      Generic.new(scheme, userinfo, host, port, 
+                  registry, path, opaque, query, 
+                  fragment)
+    end
   end
 
   #
@@ -650,7 +517,11 @@ module URI
   #   # => #<URI::HTTP:0x2022ac02 URL:http://localhost/main.rbx>
   #
   def self.join(*str)
-    DEFAULT_PARSER.join(*str)
+    u = self.parse(str[0])
+    str[1 .. -1].each do |x|
+      u = u.merge(x)
+    end
+    u
   end
 
   #
@@ -660,7 +531,7 @@ module URI
   #
   # == Args
   #
-  # +str+::
+  # +str+:: 
   #   String to extract URIs from.
   # +schemes+::
   #   Limit URI matching to a specific schemes.
@@ -678,7 +549,14 @@ module URI
   #   # => ["http://foo.example.com/bla", "mailto:test@example.com"]
   #
   def self.extract(str, schemes = nil, &block)
-    DEFAULT_PARSER.extract(str, schemes, &block)
+    if block_given?
+      str.scan(regexp(schemes)) { yield $& }
+      nil
+    else
+      result = []
+      str.scan(regexp(schemes)) { result.push $& }
+      result
+    end
   end
 
   #
@@ -688,32 +566,36 @@ module URI
   #
   # == Args
   #
-  # +match_schemes+::
+  # +match_schemes+:: 
   #   Array of schemes. If given, resulting regexp matches to URIs
   #   whose scheme is one of the match_schemes.
-  #
+  # 
   # == Description
   # Returns a Regexp object which matches to URI-like strings.
   # The Regexp object returned by this method includes arbitrary
   # number of capture group (parentheses).  Never rely on it's number.
-  #
+  # 
   # == Usage
   #
   #   require 'uri'
   #
   #   # extract first URI from html_string
   #   html_string.slice(URI.regexp)
-  #
+  # 
   #   # remove ftp URIs
   #   html_string.sub(URI.regexp(['ftp'])
-  #
+  # 
   #   # You should not rely on the number of parentheses
   #   html_string.scan(URI.regexp) do |*matches|
   #     p $&
   #   end
   #
   def self.regexp(schemes = nil)
-    DEFAULT_PARSER.make_regexp(schemes)
+    unless schemes
+      ABS_URI_REF
+    else
+      /(?=#{Regexp.union(*schemes)}:)#{PATTERN::X_ABS_URI}/xn
+    end
   end
 
 end
