@@ -16,12 +16,6 @@ class Addrinfo
       raise ArgumentError, "no address specified"
     elsif Addrinfo === args.first
       raise ArgumentError, "too many arguments" if args.length != 1
-      addrinfo = args.first
-      if (self.pfamily != addrinfo.pfamily) ||
-         (self.socktype != addrinfo.socktype)
-        raise ArgumentError, "Addrinfo type mismatch"
-      end
-      addrinfo
     elsif self.ip?
       raise ArgumentError, "IP address needs host and port but #{args.length} arguments given" if args.length != 2
       host, port = args
@@ -37,33 +31,16 @@ class Addrinfo
 
   # creates a new Socket connected to the address of +local_addrinfo+.
   #
-  # If _local_addrinfo_ is nil, the address of the socket is not bound.
-  #
-  # The _timeout_ specify the seconds for timeout.
-  # Errno::ETIMEDOUT is raised when timeout occur.
+  # If no arguments are given, the address of the socket is not bound.
   #
   # If a block is given the created socket is yielded for each address.
   #
-  def connect_internal(local_addrinfo, timeout=nil) # :yields: socket
+  def connect_internal(local_addrinfo) # :yields: socket
     sock = Socket.new(self.pfamily, self.socktype, self.protocol)
     begin
       sock.ipv6only! if self.ipv6?
       sock.bind local_addrinfo if local_addrinfo
-      if timeout
-        begin
-          sock.connect_nonblock(self)
-        rescue IO::WaitWritable
-          if !IO.select(nil, [sock], nil, timeout)
-            raise Errno::ETIMEDOUT, 'user specified timeout'
-          end
-          begin
-            sock.connect_nonblock(self) # check connection failure
-          rescue Errno::EISCONN
-          end
-        end
-      else
-        sock.connect(self)
-      end
+      sock.connect(self)
     rescue Exception
       sock.close
       raise
@@ -80,22 +57,13 @@ class Addrinfo
   end
   private :connect_internal
 
-  # :call-seq:
-  #   addrinfo.connect_from([local_addr_args], [opts]) {|socket| ... }
-  #   addrinfo.connect_from([local_addr_args], [opts])
-  #
   # creates a socket connected to the address of self.
   #
   # If one or more arguments given as _local_addr_args_,
   # it is used as the local address of the socket.
   # _local_addr_args_ is given for family_addrinfo to obtain actual address.
   #
-  # If _local_addr_args_ is not given, the local address of the socket is not bound.
-  #
-  # The optional last argument _opts_ is options represented by a hash.
-  # _opts_ may have following options:
-  #
-  # [:timeout] specify the timeout in seconds.
+  # If no arguments given, the local address of the socket is not bound.
   #
   # If a block is given, it is called with the socket and the value of the block is returned.
   # The socket is returned otherwise.
@@ -111,23 +79,12 @@ class Addrinfo
   #     puts s.read
   #   }
   #
-  def connect_from(*args, &block)
-    opts = Hash === args.last ? args.pop : {}
-    local_addr_args = args
-    connect_internal(family_addrinfo(*local_addr_args), opts[:timeout], &block)
+  def connect_from(*local_addr_args, &block)
+    connect_internal(family_addrinfo(*local_addr_args), &block)
   end
 
-  # :call-seq:
-  #   addrinfo.connect([opts]) {|socket| ... }
-  #   addrinfo.connect([opts])
-  #
   # creates a socket connected to the address of self.
   #
-  # The optional argument _opts_ is options represented by a hash.
-  # _opts_ may have following options:
-  #
-  # [:timeout] specify the timeout in seconds.
-  #
   # If a block is given, it is called with the socket and the value of the block is returned.
   # The socket is returned otherwise.
   #
@@ -136,21 +93,12 @@ class Addrinfo
   #     puts s.read
   #   }
   #
-  def connect(opts={}, &block)
-    connect_internal(nil, opts[:timeout], &block)
+  def connect(&block)
+    connect_internal(nil, &block)
   end
 
-  # :call-seq:
-  #   addrinfo.connect_to([remote_addr_args], [opts]) {|socket| ... }
-  #   addrinfo.connect_to([remote_addr_args], [opts])
-  #
   # creates a socket connected to _remote_addr_args_ and bound to self.
   #
-  # The optional last argument _opts_ is options represented by a hash.
-  # _opts_ may have following options:
-  #
-  # [:timeout] specify the timeout in seconds.
-  #
   # If a block is given, it is called with the socket and the value of the block is returned.
   # The socket is returned otherwise.
   #
@@ -159,11 +107,9 @@ class Addrinfo
   #     puts s.read
   #   }
   #
-  def connect_to(*args, &block)
-    opts = Hash === args.last ? args.pop : {}
-    remote_addr_args = args
+  def connect_to(*remote_addr_args, &block)
     remote_addrinfo = family_addrinfo(*remote_addr_args)
-    remote_addrinfo.send(:connect_internal, self, opts[:timeout], &block)
+    remote_addrinfo.send(:connect_internal, self, &block)
   end
 
   # creates a socket bound to self.
@@ -198,7 +144,7 @@ class Addrinfo
   end
 
   # creates a listening socket bound to self.
-  def listen(backlog=Socket::SOMAXCONN)
+  def listen(backlog=5)
     sock = Socket.new(self.pfamily, self.socktype, self.protocol)
     begin
       sock.ipv6only! if self.ipv6?
@@ -285,29 +231,15 @@ class Socket < BasicSocket
     end
   end
 
-  # :call-seq:
-  #   Socket.tcp(host, port, local_host=nil, local_port=nil, [opts]) {|socket| ... }
-  #   Socket.tcp(host, port, local_host=nil, local_port=nil, [opts])
-  #
   # creates a new socket object connected to host:port using TCP/IP.
   #
   # If local_host:local_port is given,
   # the socket is bound to it.
   #
-  # The optional last argument _opts_ is options represented by a hash.
-  # _opts_ may have following options:
-  #
-  # [:connect_timeout] specify the timeout in seconds.
-  #
   # If a block is given, the block is called with the socket.
   # The value of the block is returned.
   # The socket is closed when this method returns.
   #
-  # The optional last argument _opts_ is options represented by a hash.
-  # _opts_ may have following options:
-  #
-  # [:timeout] specify the timeout in seconds.
-  #
   # If no block is given, the socket is returned.
   #
   #   Socket.tcp("www.ruby-lang.org", 80) {|sock|
@@ -316,15 +248,10 @@ class Socket < BasicSocket
   #     puts sock.read
   #   }
   #
-  def self.tcp(host, port, *rest) # :yield: socket
-    opts = Hash === rest.last ? rest.pop : {}
-    raise ArgumentError, "wrong number of arguments (#{rest.length} for 2)" if 2 < rest.length
-    local_host, local_port = rest
+  def self.tcp(host, port, local_host=nil, local_port=nil) # :yield: socket
     last_error = nil
     ret = nil
 
-    connect_timeout = opts[:connect_timeout]
-
     local_addr_list = nil
     if local_host != nil || local_port != nil
       local_addr_list = Addrinfo.getaddrinfo(local_host, local_port, nil, :STREAM, nil)
@@ -338,9 +265,7 @@ class Socket < BasicSocket
         local_addr = nil
       end
       begin
-        sock = local_addr ?
-          ai.connect_from(local_addr, :timeout => connect_timeout) :
-          ai.connect(:timeout => connect_timeout)
+        sock = local_addr ? ai.connect_from(local_addr) : ai.connect
       rescue SystemCallError
         last_error = $!
         next
@@ -408,7 +333,7 @@ class Socket < BasicSocket
     sockets = ip_sockets_port0(ai_list, true)
     begin
       sockets.each {|s|
-        s.listen(Socket::SOMAXCONN)
+        s.listen(5)
       }
     rescue Exception
       sockets.each {|s| s.close }
@@ -431,7 +356,7 @@ class Socket < BasicSocket
   # The value of the block is returned.
   # The socket is closed when this method returns.
   #
-  # If _port_ is 0, actual port number is chosen dynamically.
+  # If _port_ is 0, actual port number is choosen dynamically.
   # However all sockets in the result has same port number.
   #
   #   # tcp_server_sockets returns two sockets.
@@ -443,7 +368,7 @@ class Socket < BasicSocket
   #   #=> #<Addrinfo: [::]:1296 TCP>
   #   #   #<Addrinfo: 0.0.0.0:1296 TCP>
   #
-  #   # IPv6 and IPv4 socket has same port number, 53114, even if it is chosen dynamically.
+  #   # IPv6 and IPv4 socket has same port number, 53114, even if it is choosen dynamically.
   #   sockets = Socket.tcp_server_sockets(0)
   #   sockets.each {|s| p s.local_address }
   #   #=> #<Addrinfo: [::]:53114 TCP>
@@ -574,8 +499,8 @@ class Socket < BasicSocket
   # The value of the block is returned.
   # The sockets are closed when this method returns.
   #
-  # If _port_ is zero, some port is chosen.
-  # But the chosen port is used for the all sockets.
+  # If _port_ is zero, some port is choosen.
+  # But the choosen port is used for the all sockets.
   #
   #   # UDP/IP echo server
   #   Socket.udp_server_sockets(0) {|sockets|
@@ -737,9 +662,9 @@ class Socket < BasicSocket
 
   # UDP/IP address information used by Socket.udp_server_loop.
   class UDPSource
-    # +remote_address+ is an Addrinfo object.
+    # +remote_adress+ is an Addrinfo object.
     #
-    # +local_address+ is an Addrinfo object.
+    # +local_adress+ is an Addrinfo object.
     #
     # +reply_proc+ is a Proc used to send reply back to the source.
     def initialize(remote_address, local_address, &reply_proc)
