@@ -1,19 +1,21 @@
 require File.dirname(__FILE__) + "/../spec_helper"
+require 'rational'
+require 'complex'
 
-import "java_integration.fixtures.CoreTypeMethods"
-import "java_integration.fixtures.JavaFields"
-import "java_integration.fixtures.ValueReceivingInterface"
-import "java_integration.fixtures.ValueReceivingInterfaceHandler"
+java_import "java_integration.fixtures.CoreTypeMethods"
+java_import "java_integration.fixtures.JavaFields"
+java_import "java_integration.fixtures.ValueReceivingInterface"
+java_import "java_integration.fixtures.ValueReceivingInterfaceHandler"
 
-import "java_integration.fixtures.PackageConstructor"
-import "java_integration.fixtures.ProtectedConstructor"
-import "java_integration.fixtures.PrivateConstructor"
+java_import "java_integration.fixtures.PackageConstructor"
+java_import "java_integration.fixtures.ProtectedConstructor"
+java_import "java_integration.fixtures.PrivateConstructor"
 
 describe "Java String and primitive-typed methods" do
-  it "should coerce to Ruby types when returned" do 
+  it "should coerce to Ruby types when returned" do
     CoreTypeMethods.getString.should be_kind_of(String)
     CoreTypeMethods.getString.should == "foo";
-    
+
     CoreTypeMethods.getByte.should be_kind_of(Fixnum)
     CoreTypeMethods.getByte.should == 1
     CoreTypeMethods.getShort.should be_kind_of(Fixnum)
@@ -24,34 +26,34 @@ describe "Java String and primitive-typed methods" do
     CoreTypeMethods.getInt.should == 4
     CoreTypeMethods.getLong.should be_kind_of(Fixnum)
     CoreTypeMethods.getLong.should == 8
-    
+
     CoreTypeMethods.getFloat.should be_kind_of(Float)
     CoreTypeMethods.getFloat.should == 4.5
     CoreTypeMethods.getDouble.should be_kind_of(Float)
     CoreTypeMethods.getDouble.should == 8.5
-    
+
     CoreTypeMethods.getBooleanTrue.should be_kind_of(TrueClass)
     CoreTypeMethods.getBooleanTrue.should == true
     CoreTypeMethods.getBooleanFalse.should be_kind_of(FalseClass)
     CoreTypeMethods.getBooleanFalse.should == false
-    
+
     CoreTypeMethods.getNull.should be_kind_of(NilClass)
     CoreTypeMethods.getNull.should == nil
-    
+
     CoreTypeMethods.getVoid.should == nil
 
     CoreTypeMethods.getBigInteger.should == 1234567890123456789012345678901234567890
   end
-  
+
   it "should be coerced from Ruby types when passing parameters" do
     CoreTypeMethods.setString("string").should == "string"
-    
+
     CoreTypeMethods.setByte(1).should == "1"
     CoreTypeMethods.setShort(1).should == "1"
     CoreTypeMethods.setChar(1).should == "\001"
     CoreTypeMethods.setInt(1).should == "1"
     CoreTypeMethods.setLong(1).should == "1"
-    
+
     CoreTypeMethods.setFloat(1).should == "1.0"
     CoreTypeMethods.setDouble(1).should == "1.0"
 
@@ -63,7 +65,7 @@ describe "Java String and primitive-typed methods" do
 
     CoreTypeMethods.setFloat(1.5).should == "1.5"
     CoreTypeMethods.setDouble(1.5).should == "1.5"
-    
+
     CoreTypeMethods.setBooleanTrue(true).should == "true"
     CoreTypeMethods.setBooleanFalse(false).should == "false"
 
@@ -90,6 +92,8 @@ describe "Java String and primitive-typed methods" do
 
     CoreTypeMethods.setFloatObj(1).should == "1.0"
     CoreTypeMethods.setDoubleObj(1).should == "1.0"
+    CoreTypeMethods.setNumber(1).should == "java.lang.Long"
+    CoreTypeMethods.setSerializable(1).should == "java.lang.Long"
 
     CoreTypeMethods.setByteObj(1.5).should == "1"
     CoreTypeMethods.setShortObj(1.5).should == "1"
@@ -99,6 +103,8 @@ describe "Java String and primitive-typed methods" do
 
     CoreTypeMethods.setFloatObj(1.5).should == "1.5"
     CoreTypeMethods.setDoubleObj(1.5).should == "1.5"
+    CoreTypeMethods.setNumber(1.5).should == "java.lang.Double"
+    CoreTypeMethods.setSerializable(1.5).should == "java.lang.Double"
 
     CoreTypeMethods.setBooleanTrueObj(true).should == "true"
     CoreTypeMethods.setBooleanFalseObj(false).should == "false"
@@ -114,10 +120,35 @@ describe "Java String and primitive-typed methods" do
 
     CoreTypeMethods.setBooleanTrueObj(nil).should == "null"
     CoreTypeMethods.setBooleanFalseObj(nil).should == "null"
-    
+
     CoreTypeMethods.setNull(nil).should == "null"
   end
-  
+
+  it "coerce from boxed Java types to primitives when passing parameters" do
+    CoreTypeMethods.setString("string".to_java(:string)).should == "string"
+
+    CoreTypeMethods.setByte(1.to_java(:byte)).should == "1"
+    CoreTypeMethods.setShort(1.to_java(:short)).should == "1"
+    CoreTypeMethods.setChar(1.to_java(:char)).should == "\001"
+    CoreTypeMethods.setInt(1.to_java(:int)).should == "1"
+    CoreTypeMethods.setLong(1.to_java(:long)).should == "1"
+
+    CoreTypeMethods.setFloat(1.to_java(:float)).should == "1.0"
+    CoreTypeMethods.setDouble(1.to_java(:double)).should == "1.0"
+
+    CoreTypeMethods.setByte(1.5.to_java(:byte)).should == "1"
+    CoreTypeMethods.setShort(1.5.to_java(:short)).should == "1"
+    CoreTypeMethods.setChar(1.5.to_java(:char)).should == "\001"
+    CoreTypeMethods.setInt(1.5.to_java(:int)).should == "1"
+    CoreTypeMethods.setLong(1.5.to_java(:long)).should == "1"
+
+    CoreTypeMethods.setFloat(1.5.to_java(:float)).should == "1.5"
+    CoreTypeMethods.setDouble(1.5.to_java(:double)).should == "1.5"
+
+    CoreTypeMethods.setBooleanTrue(true.to_java(:boolean)).should == "true"
+    CoreTypeMethods.setBooleanFalse(false.to_java(:boolean)).should == "false"
+  end
+
   it "should raise errors when passed values can not be precisely coerced" do
     lambda { CoreTypeMethods.setByte(1 << 8) }.should raise_error(RangeError)
     lambda { CoreTypeMethods.setShort(1 << 16) }.should raise_error(RangeError)
@@ -125,24 +156,13 @@ describe "Java String and primitive-typed methods" do
     lambda { CoreTypeMethods.setInt(1 << 32) }.should raise_error(RangeError)
     lambda { CoreTypeMethods.setLong(1 << 64) }.should raise_error(RangeError)
   end
-  
-  it "should select the most narrow and precise overloaded method" do
-    pending "selection based on precision is not supported yet" do
-      CoreTypeMethods.getType(1).should == "byte"
-      CoreTypeMethods.getType(1 << 8).should == "short"
-      CoreTypeMethods.getType(1 << 16).should == "int"
-      CoreTypeMethods.getType(1.0).should == "float"
-    end
-    CoreTypeMethods.getType(1 << 32).should == "long"
-    
-    CoreTypeMethods.getType(2.0 ** 128).should == "double"
-    
-    CoreTypeMethods.getType("foo").should == "String"
-    pending "passing null to overloaded methods randomly selects from them" do
-      CoreTypeMethods.getType(nil).should == "CharSequence"
-    end
 
-    CoreTypeMethods.getType(BigDecimal.new('1.1')).should == "BigDecimal"
+  it "should select the method that matches precision of the incoming value" do
+    CoreTypeMethods.getType(1 << 32).should == "long"
+
+    CoreTypeMethods.getType(2.0 ** 128).should == "double"
+
+    CoreTypeMethods.getType("foo").should == "String"
   end
 end
 
@@ -187,6 +207,16 @@ describe "Java Object-typed methods" do
     CoreTypeMethods.getObjectType(true).should == "class java.lang.Boolean"
     CoreTypeMethods.getObjectType(1 << 128).should == "class java.math.BigInteger"
   end
+
+  it "passes coerced to_java values - keeping the Java type" do
+    CoreTypeMethods.getObjectType('foo'.to_java).should == "class java.lang.String"
+    CoreTypeMethods.getObjectType(0.to_java).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(1.to_java(:int)).should == "class java.lang.Integer"
+    CoreTypeMethods.getObjectType(1.to_java(:byte)).should == "class java.lang.Byte"
+    CoreTypeMethods.getObjectType(0.1.to_java(:double)).should == "class java.lang.Double"
+    CoreTypeMethods.getObjectType(0.1.to_java('java.lang.Float')).should == "class java.lang.Float"
+    CoreTypeMethods.getObjectType(false.to_java).should == "class java.lang.Boolean"
+  end
 end
 
 describe "Java String and primitive-typed fields" do
@@ -194,7 +224,7 @@ describe "Java String and primitive-typed fields" do
     # static
     JavaFields.stringStaticField.should be_kind_of(String)
     JavaFields.stringStaticField.should == "foo";
-    
+
     JavaFields.byteStaticField.should be_kind_of(Fixnum)
     JavaFields.byteStaticField.should == 1
     JavaFields.shortStaticField.should be_kind_of(Fixnum)
@@ -205,29 +235,29 @@ describe "Java String and primitive-typed fields" do
     JavaFields.intStaticField.should == 4
     JavaFields.longStaticField.should be_kind_of(Fixnum)
     JavaFields.longStaticField.should == 8
-    
+
     JavaFields.floatStaticField.should be_kind_of(Float)
     JavaFields.floatStaticField.should == 4.5
     JavaFields.doubleStaticField.should be_kind_of(Float)
     JavaFields.doubleStaticField.should == 8.5
-    
+
     JavaFields.trueStaticField.should be_kind_of(TrueClass)
     JavaFields.trueStaticField.should == true
     JavaFields.falseStaticField.should be_kind_of(FalseClass)
     JavaFields.falseStaticField.should == false
-    
+
     JavaFields.nullStaticField.should be_kind_of(NilClass)
     JavaFields.nullStaticField.should == nil
 
     JavaFields.bigIntegerStaticField.should be_kind_of(Bignum)
     JavaFields.bigIntegerStaticField.should ==
       1234567890123456789012345678901234567890
-    
+
     # instance
     jf = JavaFields.new
     jf.stringField.should be_kind_of(String)
     jf.stringField.should == "foo";
-    
+
     jf.byteField.should be_kind_of(Fixnum)
     jf.byteField.should == 1
     jf.shortField.should be_kind_of(Fixnum)
@@ -238,17 +268,17 @@ describe "Java String and primitive-typed fields" do
     jf.intField.should == 4
     jf.longField.should be_kind_of(Fixnum)
     jf.longField.should == 8
-    
+
     jf.floatField.should be_kind_of(Float)
     jf.floatField.should == 4.5
     jf.doubleField.should be_kind_of(Float)
     jf.doubleField.should == 8.5
-    
+
     jf.trueField.should be_kind_of(TrueClass)
     jf.trueField.should == true
     jf.falseField.should be_kind_of(FalseClass)
     jf.falseField.should == false
-    
+
     jf.nullField.should be_kind_of(NilClass)
     jf.nullField.should == nil
 
@@ -271,17 +301,17 @@ describe "Java primitive-box-typed fields" do
     JavaFields.intObjStaticField.should == 4
     JavaFields.longObjStaticField.should be_kind_of(Fixnum)
     JavaFields.longObjStaticField.should == 8
-    
+
     JavaFields.floatObjStaticField.should be_kind_of(Float)
     JavaFields.floatObjStaticField.should == 4.5
     JavaFields.doubleObjStaticField.should be_kind_of(Float)
     JavaFields.doubleObjStaticField.should == 8.5
-    
+
     JavaFields.trueObjStaticField.should be_kind_of(TrueClass)
     JavaFields.trueObjStaticField.should == true
     JavaFields.falseObjStaticField.should be_kind_of(FalseClass)
     JavaFields.falseObjStaticField.should == false
-    
+
     # instance
     jf = JavaFields.new
     jf.byteObjField.should be_kind_of(Fixnum)
@@ -294,12 +324,12 @@ describe "Java primitive-box-typed fields" do
     jf.intObjField.should == 4
     jf.longObjField.should be_kind_of(Fixnum)
     jf.longObjField.should == 8
-    
+
     jf.floatObjField.should be_kind_of(Float)
     jf.floatObjField.should == 4.5
     jf.doubleObjField.should be_kind_of(Float)
     jf.doubleObjField.should == 8.5
-    
+
     jf.trueObjField.should be_kind_of(TrueClass)
     jf.trueObjField.should == true
     jf.falseObjField.should be_kind_of(FalseClass)
@@ -312,42 +342,42 @@ describe "Java String, primitive, and object-typed interface methods" do
     impl = Class.new {
       attr_accessor :result
       include ValueReceivingInterface
-      
+
       def receiveObject(obj)
         self.result = obj
         obj
       end
-      
+
       def receiveLongAndDouble(l, d)
         str = (l + d).to_s
         self.result = str
         str
       end
-      
+
       %w[String Byte Short Char Int Long Float Double Null True False].each do |type|
         alias_method "receive#{type}".intern, :receiveObject
       end
     }
-    
+
     vri = impl.new
     vri_handler = ValueReceivingInterfaceHandler.new(vri);
-    
+
     obj = java.lang.Object.new
     vri_handler.receiveObject(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == java.lang.Object
-    
+
     obj = "foo"
     vri_handler.receiveString(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == String
-    
+
     obj = 1
-    
+
     vri_handler.receiveByte(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Fixnum
-    
+
     vri_handler.receiveShort(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Fixnum
@@ -355,35 +385,35 @@ describe "Java String, primitive, and object-typed interface methods" do
     vri_handler.receiveChar(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Fixnum
-    
+
     vri_handler.receiveInt(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Fixnum
-    
+
     vri_handler.receiveLong(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Fixnum
-    
+
     vri_handler.receiveFloat(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Float
-    
+
     vri_handler.receiveDouble(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Float
-    
+
     vri_handler.receiveNull(nil).should == nil
     vri.result.should == nil
     vri.result.class.should == NilClass
-    
+
     vri_handler.receiveTrue(true).should == true
     vri.result.should == true
     vri.result.class.should == TrueClass
-    
+
     vri_handler.receiveFalse(false).should == false
     vri.result.should == false
     vri.result.class.should == FalseClass
-    
+
     vri_handler.receiveLongAndDouble(1, 1.0).should == "2.0"
     vri.result.should == "2.0"
     vri.result.class.should == String
@@ -405,7 +435,7 @@ describe "Java primitive-typed interface methods" do
         alias_method "receive#{type}".intern, :receive_primitive
       end
     }
-    
+
     vri = impl.new
     vri_handler = ValueReceivingInterfaceHandler.new(vri);
 
@@ -602,7 +632,7 @@ describe "Fixnum\#to_java" do
     float.class.should == java.lang.Float
     double.class.should == java.lang.Double
   end
-  
+
   it "coerces to java.lang.Long when asked to coerce to java.lang.Object" do
     obj = 123.to_java java.lang.Object
     obj2 = 123.to_java :object
@@ -703,7 +733,7 @@ describe "Class\#to_java" do
     end
 
     it "returns reified class for reified used classes" do
-      rubycls = Class.new
+      rubycls = Class.new; require 'jruby/core_ext'
       rubycls.become_java!
       rubycls.to_java(cls).should == JRuby.reference(rubycls).reified_class
     end
@@ -714,21 +744,27 @@ describe "Class\#to_java" do
   end
 
   describe "when passed java.lang.Object.class" do
-    cls = java.lang.Object
     it "coerces core classes to their Ruby class object" do
-      # TODO: add all core, native types here
-      [Object, Array, String, Hash, File, IO].each do |rubycls|
-        rubycls.to_java(cls).should == rubycls
+      [Object, Array, String, Hash, File, IO].each do |clazz|
+        clazz.to_java(java.lang.Object).should == clazz
       end
+      BasicObject.to_java(java.lang.Object).should == BasicObject if defined? BasicObject
+      [Bignum, Dir, ENV, FalseClass, Fixnum, Float, Kernel, Struct, Symbol, Thread].each do |clazz|
+        clazz.to_java(java.lang.Object).should == clazz
+      end
+      Exception.to_java(java.lang.Object).should == Exception
+      StandardError.to_java(java.lang.Object).should == StandardError
     end
 
-    it "coerces user classes to their Ruby class object" do
-      rubycls = Class.new
-      rubycls.to_java(cls).should == rubycls;
+    it "coerces user classes/modules to their Ruby class object" do
+      clazz = Class.new
+      clazz.to_java(java.lang.Object).should == clazz
+      clazz = Module.new
+      clazz.to_java(java.lang.Object).should == clazz
     end
 
     it "converts Java proxy classes to their proxy class (Ruby class) equivalent" do
-      java.util.ArrayList.to_java(cls).should == java.util.ArrayList
+      java.util.ArrayList.to_java(java.lang.Object).should == java.util.ArrayList
     end
   end
 end
@@ -789,4 +825,36 @@ describe "Time\"to_java" do
       d.class.should == java.util.Date
     end
   end
-end
\ No newline at end of file
+end
+
+describe "A Rational object" do
+  before :each do
+    @rational = Rational(1,2)
+  end
+
+  it "is left uncoerced with to_java" do
+    @rational.to_java.should == @rational
+  end
+
+  it "fails to coerce to types not assignable from the given type" do
+    lambda do
+      @rational.to_java(java.lang.String)
+    end.should raise_error
+  end
+end
+
+describe "A Complex object" do
+  before :each do
+    @complex = Complex(1,2)
+  end
+
+  it "is left uncoerced with to_java" do
+    @complex.to_java.should == @complex
+  end
+
+  it "fails to coerce to types not assignable from the given type" do
+    lambda do
+      @complex.to_java(java.lang.String)
+    end.should raise_error
+  end
+end
