@@ -1,3 +1,6 @@
+windows = RbConfig::CONFIG['host_os'] =~ /mswin|mingw/
+
+exclude :test_export_format_is_DSA_PUBKEY_pem, 'most likely \r\n' if windows
 exclude :test_new, 'needs investigation'
 exclude :test_private, 'needs investigation'
 exclude :test_read_DSAPublicKey_pem, 'needs investigation'
