@@ -9,5 +9,3 @@ exclude :test_noname_unpack_sockaddr_un, "needs investigation"
 exclude :test_nul, "needs investigation"
 exclude :test_socket_pair_with_block, "needs investigation"
 exclude :test_too_long_path, "needs investigation"
-exclude :test_unix_server_socket, "needs investigation"
-exclude :test_unix_socket_pair_with_block, "needs investigation"
\ No newline at end of file
