@@ -6,9 +6,9 @@ require 'tempfile'
 class TestLoggerSeverity < Test::Unit::TestCase
   def test_enum
     logger_levels = Logger.constants
-    levels = [:WARN, :UNKNOWN, :INFO, :FATAL, :DEBUG, :ERROR]
+    levels = ["WARN", "UNKNOWN", "INFO", "FATAL", "DEBUG", "ERROR"]
     Logger::Severity.constants.each do |level|
-      assert(levels.include?(level))
+      assert(levels.include?(level.to_s))
       assert(logger_levels.include?(level))
     end
     assert_equal(levels.size, Logger::Severity.constants.size)
@@ -23,14 +23,10 @@ class TestLogger < Test::Unit::TestCase
     @logger = Logger.new(nil)
   end
 
-  def test_const_progname
-    assert %r!\Alogger\.rb/\S+\z! === Logger::ProgName
-  end
-
   class Log
     attr_reader :label, :datetime, :pid, :severity, :progname, :msg
     def initialize(line)
-      /\A(\w+), \[([^#]*)#(\d+)\]\s+(\w+) -- (\w*): ([\x0-\xff]*)/n =~ line
+      /\A(\w+), \[([^#]*)#(\d+)\]\s+(\w+) -- (\w*): ([\x0-\xff]*)/ =~ line
       @label, @datetime, @pid, @severity, @progname, @msg = $1, $2, $3, $4, $5, $6
     end
   end
@@ -49,6 +45,7 @@ class TestLogger < Test::Unit::TestCase
     logger.__send__(msg_id, *arg, &block)
     logdev.open
     msg = logdev.read
+    msg.gsub!(/\r\n/, "\n") # tempfile writes with crlf logic on...fix for tests    msg
     logdev.close
     msg
   end
@@ -262,27 +259,60 @@ class TestLogger < Test::Unit::TestCase
 end
 
 class TestLogDevice < Test::Unit::TestCase
-  def d(log)
-    Logger::LogDevice.new(log)
+  class LogExcnRaiser
+    def write(*arg)
+      raise 'disk is full'
+    end
+
+    def close
+    end
+
+    def stat
+      Object.new
+    end
+  end
+
+  def setup
+    @tempfile = Tempfile.new("logger")
+    @tempfile.close
+    @filename = @tempfile.path
+    File.unlink(@filename)
+  end
+
+  def teardown
+    @tempfile.close(true)
+  end
+
+  def d(log, opt = {})
+    Logger::LogDevice.new(log, opt)
   end
 
   def test_initialize
     logdev = d(STDERR)
     assert_equal(STDERR, logdev.dev)
     assert_nil(logdev.filename)
-    assert_raise(TypeError) do
+    assert_raises(TypeError) do
       d(nil)
     end
     #
-    filename = __FILE__ + ".#{$$}"
+    logdev = d(@filename)
     begin
-      logdev = d(filename)
-      assert(File.exist?(filename))
+      assert(File.exist?(@filename))
       assert(logdev.dev.sync)
-      assert_equal(filename, logdev.filename)
+      assert_equal(@filename, logdev.filename)
+      logdev.write('hello')
+    ensure
+      logdev.close
+    end
+    # create logfile whitch is already exist.
+    logdev = d(@filename)
+    begin
+      logdev.write('world')
+      logfile = File.read(@filename)
+      assert_equal(2, logfile.split(/\n/).size)
+      assert_match(/^helloworld$/, logfile)
     ensure
       logdev.close
-      File.unlink(filename)
     end
   end
 
@@ -295,6 +325,21 @@ class TestLogDevice < Test::Unit::TestCase
     msg = r.read
     r.close
     assert_equal("msg2\n\n", msg)
+    #
+    logdev = d(LogExcnRaiser.new)
+    class << (stderr = '')
+      alias write <<
+    end
+    $stderr, stderr = stderr, $stderr
+    begin
+      assert_nothing_raised do
+        logdev.write('hello')
+      end
+    ensure
+      logdev.close
+      $stderr, stderr = stderr, $stderr
+    end
+    assert_equal "log writing failed. disk is full\n", stderr
   end
 
   def test_close
@@ -377,4 +422,102 @@ class TestLogDevice < Test::Unit::TestCase
     File.unlink(logfile1)
     File.unlink(logfile2)
   end
+
+  def test_shifting_age_variants
+    logger = Logger.new(@filename, 'daily')
+    logger.info('daily')
+    logger.close
+    logger = Logger.new(@filename, 'weekly')
+    logger.info('weekly')
+    logger.close
+    logger = Logger.new(@filename, 'monthly')
+    logger.info('monthly')
+    logger.close
+  end
+
+  def test_shifting_age
+    # shift_age other than 'daily', 'weekly', and 'monthly' means 'everytime'
+    yyyymmdd = Time.now.strftime("%Y%m%d")
+    filename1 = @filename + ".#{yyyymmdd}"
+    filename2 = @filename + ".#{yyyymmdd}.1"
+    filename3 = @filename + ".#{yyyymmdd}.2"
+    begin
+      logger = Logger.new(@filename, 'now')
+      assert(File.exist?(@filename))
+      assert(!File.exist?(filename1))
+      assert(!File.exist?(filename2))
+      assert(!File.exist?(filename3))
+      logger.info("0" * 15)
+      assert(File.exist?(@filename))
+      assert(File.exist?(filename1))
+      assert(!File.exist?(filename2))
+      assert(!File.exist?(filename3))
+      logger.warn("0" * 15)
+      assert(File.exist?(@filename))
+      assert(File.exist?(filename1))
+      assert(File.exist?(filename2))
+      assert(!File.exist?(filename3))
+      logger.error("0" * 15)
+      assert(File.exist?(@filename))
+      assert(File.exist?(filename1))
+      assert(File.exist?(filename2))
+      assert(File.exist?(filename3))
+    ensure
+      logger.close if logger
+      [filename1, filename2, filename3].each do |filename|
+        File.unlink(filename) if File.exist?(filename)
+      end
+    end
+  end
+end
+
+
+class TestLoggerApplication < Test::Unit::TestCase
+  def setup
+    @app = Logger::Application.new('appname')
+    @tempfile = Tempfile.new("logger")
+    @tempfile.close
+    @filename = @tempfile.path
+    File.unlink(@filename)
+  end
+
+  def teardown
+    @tempfile.close(true)
+  end
+
+  def test_initialize
+    app = Logger::Application.new('appname')
+    assert_equal('appname', app.appname)
+  end
+
+  def test_start
+    @app.set_log(@filename)
+    begin
+      @app.level = Logger::UNKNOWN
+      @app.start # logs FATAL log
+      assert_equal(1, File.read(@filename).split(/\n/).size)
+    ensure
+      @app.logger.close
+    end
+  end
+
+  def test_logger
+    @app.level = Logger::WARN
+    @app.set_log(@filename)
+    begin
+      assert_equal(Logger::WARN, @app.logger.level)
+    ensure
+      @app.logger.close
+    end
+    @app.logger = logger = Logger.new(STDOUT)
+    assert_equal(logger, @app.logger)
+    assert_equal(Logger::WARN, @app.logger.level)
+    @app.log = @filename
+    begin
+      assert(logger != @app.logger)
+      assert_equal(Logger::WARN, @app.logger.level)
+    ensure
+      @app.logger.close
+    end
+  end
 end
