@@ -2,7 +2,7 @@
 #												vim:sw=4:ts=4
 # $Id$
 #
-require 'psych/helper'
+require_relative 'helper'
 require 'ostruct'
 
 # [ruby-core:01946]
@@ -27,6 +27,10 @@ class Psych_Unit_Tests < Psych::TestCase
       assert_match "2010-10-10 00:00:00.000000000 Z", yaml
     end
 
+    def test_multiline_regexp
+        assert_cycle(Regexp.new("foo\nbar"))
+    end
+
     # [ruby-core:34969]
     def test_regexp_with_n
         assert_cycle(Regexp.new('',0,'n'))
@@ -250,7 +254,6 @@ EOY
 
 	def test_spec_mapping_between_sequences
 		# Complex key #1
-		dj = Date.new( 2001, 7, 23 )
 		assert_parse_only(
 			{ [ 'Detroit Tigers', 'Chicago Cubs' ] => [ Date.new( 2001, 7, 23 ) ],
 			  [ 'New York Yankees', 'Atlanta Braves' ] => [ Date.new( 2001, 7, 2 ), Date.new( 2001, 8, 12 ), Date.new( 2001, 8, 14 ) ] }, <<EOY
@@ -606,7 +609,7 @@ EOY
 	def test_spec_domain_prefix
         customer_proc = proc { |type, val|
             if Hash === val
-                scheme, domain, type = type.split( ':', 3 )
+                _, _, type = type.split( ':', 3 )
                 val['type'] = "domain #{type}"
                 val
             else
