@@ -178,7 +178,7 @@ class TestProc < Test::Unit::TestCase
     b = proc { :foo }
     assert_equal(:foo, b.curry[])
 
-    b = lambda {|x, y, &b| b.call(x + y) }.curry
+    b = lambda {|x, y, &blk| blk.call(x + y) }.curry
     b = b.call(2) { raise }
     b = b.call(3) {|x| x + 4 }
     assert_equal(9, b)
@@ -379,7 +379,7 @@ class TestProc < Test::Unit::TestCase
     assert_equal [1,2,3,[4,5,6]], pr.call([1,2,3,4,5,6])
 
     r = proc{|*a| a}.call([1,2,3])
-    assert [1,2,3], r
+    assert_equal [[1,2,3]], r
   end
 
   def test_proc_args_rest_and_post
@@ -620,7 +620,7 @@ class TestProc < Test::Unit::TestCase
     assert_equal [1, 2, 3, 4, 5, 6, Proc, :x], (pr.call(1, 2, 3, 4, 5, 6, 7){|x| x})
   end
 
-  def test_proc_args_opt_and_block
+  def test_proc_args_opt_and_block2
     pr = proc {|a,b,c=:c,d=:d,*e,&f|
       [a, b, c, d, e, f.class, f&&f.call(:x)]
     }
@@ -706,7 +706,7 @@ class TestProc < Test::Unit::TestCase
     assert_equal([[:opt, nil], [:block, :b]], proc {|(a), &b|}.parameters)
     assert_equal([[:opt, :a], [:opt, :b], [:opt, :c], [:opt, :d], [:rest, :e], [:opt, :f], [:opt, :g], [:block, :h]], proc {|a,b,c=:c,d=:d,*e,f,g,&h|}.parameters)
 
-    assert_equal([[:req]], method(:require).parameters)
+    assert_equal([[:req]], method(:putc).parameters)
     assert_equal([[:rest]], method(:p).parameters)
   end
 
@@ -735,6 +735,10 @@ class TestProc < Test::Unit::TestCase
     assert_equal([[:req, :a], [:rest, :b], [:req, :c], [:block, :d]], method(:pmo6).to_proc.parameters)
     assert_equal([[:req, :a], [:opt, :b], [:rest, :c], [:req, :d], [:block, :e]], method(:pmo7).to_proc.parameters)
     assert_equal([[:req], [:block, :b]], method(:pma1).to_proc.parameters)
+
+    assert_equal([], "".method(:upcase).to_proc.parameters)
+    assert_equal([[:rest]], "".method(:gsub).to_proc.parameters)
+    assert_equal([[:rest]], proc {}.curry.parameters)
   end
 
   def test_to_s
@@ -746,20 +750,72 @@ class TestProc < Test::Unit::TestCase
     assert(x.to_s.tainted?)
   end
 
-  def source_location_test
-    __LINE__
+  @@line_of_source_location_test = __LINE__ + 1
+  def source_location_test a=1,
+    b=2
   end
 
   def test_source_location
     file, lineno = method(:source_location_test).source_location
     assert_match(/^#{ Regexp.quote(__FILE__) }$/, file)
-    assert_equal(source_location_test - 1, lineno)
+    assert_equal(@@line_of_source_location_test, lineno, 'Bug #2427')
+  end
+
+  @@line_of_attr_reader_source_location_test   = __LINE__ + 3
+  @@line_of_attr_writer_source_location_test   = __LINE__ + 3
+  @@line_of_attr_accessor_source_location_test = __LINE__ + 3
+  attr_reader   :attr_reader_source_location_test
+  attr_writer   :attr_writer_source_location_test
+  attr_accessor :attr_accessor_source_location_test
+
+  def test_attr_source_location
+    file, lineno = method(:attr_reader_source_location_test).source_location
+    assert_match(/^#{ Regexp.quote(__FILE__) }$/, file)
+    assert_equal(@@line_of_attr_reader_source_location_test, lineno)
+
+    file, lineno = method(:attr_writer_source_location_test=).source_location
+    assert_match(/^#{ Regexp.quote(__FILE__) }$/, file)
+    assert_equal(@@line_of_attr_writer_source_location_test, lineno)
+
+    file, lineno = method(:attr_accessor_source_location_test).source_location
+    assert_match(/^#{ Regexp.quote(__FILE__) }$/, file)
+    assert_equal(@@line_of_attr_accessor_source_location_test, lineno)
+
+    file, lineno = method(:attr_accessor_source_location_test=).source_location
+    assert_match(/^#{ Regexp.quote(__FILE__) }$/, file)
+    assert_equal(@@line_of_attr_accessor_source_location_test, lineno)
   end
 
   def test_splat_without_respond_to
-    def (obj = Object.new).respond_to?(m); false end
+    def (obj = Object.new).respond_to?(m,*); false end
     [obj].each do |a, b|
       assert_equal([obj, nil], [a, b], '[ruby-core:24139]')
     end
   end
+
+  def test_curry_with_trace
+    bug3751 = '[ruby-core:31871]'
+    set_trace_func(proc {})
+    test_curry
+  ensure
+    set_trace_func(nil)
+  end
+
+  def test_block_propagation
+    bug3792 = '[ruby-core:32075]'
+    c = Class.new do
+      def foo
+        yield
+      end
+    end
+
+    o = c.new
+    f = :foo.to_proc
+    assert_nothing_raised(LocalJumpError, bug3792) {
+      assert_equal('bar', f.(o) {'bar'}, bug3792)
+    }
+    assert_nothing_raised(LocalJumpError, bug3792) {
+      assert_equal('zot', o.method(:foo).to_proc.() {'zot'}, bug3792)
+    }
+  end
 end
