@@ -1,8 +1,8 @@
-require_relative 'gem_installer_test_case'
+require 'rubygems/installer_test_case'
 require 'rubygems/install_update_options'
 require 'rubygems/command'
 
-class TestGemInstallUpdateOptions < GemInstallerTestCase
+class TestGemInstallUpdateOptions < Gem::InstallerTestCase
 
   def setup
     super
@@ -13,17 +13,12 @@ class TestGemInstallUpdateOptions < GemInstallerTestCase
   end
 
   def test_add_install_update_options
-    args = %w[-i /install_to --rdoc --ri -E -f -t -w -P HighSecurity
+    args = %w[-i /install_to --rdoc --ri -E -f -w -P HighSecurity
               --ignore-dependencies --format-exec --include-dependencies]
 
     assert @cmd.handles?(args)
   end
 
-  def test_prerelease
-    @cmd.handle_options %w[--prerelease]
-    assert_equal true, @cmd.options[:prerelease]
-  end
-
   def test_security_policy
     @cmd.handle_options %w[-P HighSecurity]
 
@@ -45,9 +40,8 @@ class TestGemInstallUpdateOptions < GemInstallerTestCase
 
     @installer = Gem::Installer.new @gem, @cmd.options
     @installer.install
-    assert File.exist?(File.join(Gem.user_dir, 'gems'))
-    assert File.exist?(File.join(Gem.user_dir, 'gems',
-                                 @spec.full_name))
+    assert_path_exists File.join(Gem.user_dir, 'gems')
+    assert_path_exists File.join(Gem.user_dir, 'gems', @spec.full_name)
   end
 
   def test_user_install_disabled_read_only
@@ -58,16 +52,16 @@ class TestGemInstallUpdateOptions < GemInstallerTestCase
 
       refute @cmd.options[:user_install]
 
-      File.chmod 0755, @userhome
+      FileUtils.chmod 0755, @userhome
       FileUtils.chmod 0000, @gemhome
 
+      Gem.use_paths @gemhome, @userhome
+
       assert_raises(Gem::FilePermissionError) do
-        @installer = Gem::Installer.new @gem, @cmd.options
+        Gem::Installer.new(@gem, @cmd.options).install
       end
     end
   ensure
     FileUtils.chmod 0755, @gemhome
   end
-
 end
-
