@@ -1,8 +1,8 @@
-require_relative 'gemutilities'
+require 'rubygems/test_case'
 require 'rubygems/command'
 require 'rubygems/version_option'
 
-class TestGemVersionOption < RubyGemTestCase
+class TestGemVersionOption < Gem::TestCase
 
   def setup
     super
