@@ -7,13 +7,14 @@
 
 require "scanf"
 
-class TestScanfIO
+class TestScanfIO < Test::Unit::TestCase
   def test_io
     fh = File.new(File.join(File.dirname(__FILE__), "data.txt"), "r")
     assert_equal(0, fh.pos)
     assert_equal(["this", "is"], fh.scanf("%s%s"))
-    assert_equal([33, "littel"], fh.scanf("%da fun%s"))
-    #p fh.pos
+    assert_equal([33, "little"], fh.scanf("%da fun%s"))
+  ensure
+    fh.close
   end
 end
 
