@@ -1,6 +1,7 @@
 require 'test/unit'
 
 class TestLoopStuff < Test::Unit::TestCase
+  IS19 = RUBY_VERSION =~ /1\.9/
 
   FILE = "while_tmp"
 
@@ -49,19 +50,21 @@ class TestLoopStuff < Test::Unit::TestCase
     end
   end
 
-  def testRedoWithWhile
-    File.open(FILE) do |tmp|
-      while tmp.gets()
-        line = $_
-        $_ = gsub(/vt100/, 'VT100')
-        if $_ != line
-          gsub!('VT100', 'Vt100')
-          redo;
+  unless IS19
+    def testRedoWithWhile
+      File.open(FILE) do |tmp|
+        while tmp.gets()
+          line = $_
+          $_ = gsub(/vt100/, 'VT100')
+          if $_ != line
+            gsub!('VT100', 'Vt100')
+            redo;
+          end
+          assert(/vt100/ !~ $_)
+          assert(/VT100/ !~ $_)
         end
-        assert(/vt100/ !~ $_)
-        assert(/VT100/ !~ $_)
+        assert(tmp.eof?)
       end
-      assert(tmp.eof?)
     end
   end
 
@@ -124,7 +127,7 @@ class TestLoopStuff < Test::Unit::TestCase
 
   def testNestedIterator
     i = 0
-    nestedIteratorHelper { |i| break if i == 5 }
+    nestedIteratorHelper { |i2| i = i2; break if i == 5 }
     assert_equal(5, i)
   end
 
@@ -218,47 +221,47 @@ class TestLoopStuff < Test::Unit::TestCase
 
   def testClassFullOfIterators
     x = nil
-    IterTest.new([0]).each0 { |x| x = x }
+    IterTest.new([0]).each0 { |x2| x = x2 }
     assert_equal(0, x)
-    IterTest.new([1]).each1 { |x| x = x }
+    IterTest.new([1]).each1 { |x2| x = x2 }
     assert_equal(1, x)
-    IterTest.new([2]).each2 { |x| x = x }
+    IterTest.new([2]).each2 { |x2| x = x2 }
     assert_equal([2], x)
-    IterTest.new([3]).each3 { |x| x = x }
+    IterTest.new([3]).each3 { |x2| x = x2 }
     assert_equal(3, x)
-    IterTest.new([4]).each4 { |x| x = x }
+    IterTest.new([4]).each4 { |x2| x = x2 }
     assert_equal(4, x)
-    IterTest.new([5]).each5 { |x| x = x }
+    IterTest.new([5]).each5 { |x2| x = x2 }
     assert_equal(5, x)
-    IterTest.new([6]).each6 { |x| x = x }
+    IterTest.new([6]).each6 { |x2| x = x2 }
     assert_equal([6], x)
-    IterTest.new([7]).each7 { |x| x = x }
+    IterTest.new([7]).each7 { |x2| x = x2 }
     assert_equal(7, x)
-    IterTest.new([8]).each8 { |x| x = x }
+    IterTest.new([8]).each8 { |x2| x = x2 }
     assert_equal(8, x)
     
-    IterTest.new([[0]]).each0 { |x| x = x }
+    IterTest.new([[0]]).each0 { |x2| x = x2 }
     assert_equal([0], x)
-    IterTest.new([[1]]).each1 { |x| x = x }
+    IterTest.new([[1]]).each1 { |x2| x = x2 }
     assert_equal([1], x)
-    IterTest.new([[2]]).each2 { |x| x = x }
+    IterTest.new([[2]]).each2 { |x2| x = x2 }
     assert_equal([[2]], x)
-    IterTest.new([[3]]).each3 { |x| x = x }
+    IterTest.new([[3]]).each3 { |x2| x = x2 }
     assert_equal(3, x)
-    IterTest.new([[4]]).each4 { |x| x = x }
+    IterTest.new([[4]]).each4 { |x2| x = x2 }
     assert_equal([4], x)
-    IterTest.new([[5]]).each5 { |x| x = x }
+    IterTest.new([[5]]).each5 { |x2| x = x2 }
     assert_equal([5], x)
-    IterTest.new([[6]]).each6 { |x| x = x }
+    IterTest.new([[6]]).each6 { |x2| x = x2 }
     assert_equal([[6]], x)
-    IterTest.new([[7]]).each7 { |x| x = x }
+    IterTest.new([[7]]).each7 { |x2| x = x2 }
     assert_equal(7, x)
-    IterTest.new([[8]]).each8 { |x| x = x }
+    IterTest.new([[8]]).each8 { |x2| x = x2 }
     assert_equal([8], x)
     
-    IterTest.new([[0,0]]).each0 { |x| x = x }
+    IterTest.new([[0,0]]).each0 { |x2| x = x2 }
     assert_equal([0, 0], x)
-    IterTest.new([[8,8]]).each8 { |x| x = x }
+    IterTest.new([[8,8]]).each8 { |x2| x = x2 }
     assert_equal([8, 8], x)
   end
 
