@@ -1,7 +1,9 @@
-require 'java'
 require 'test/unit'
+require 'test/test_helper'
+require 'java'
 
 class TestJavaExtension < Test::Unit::TestCase
+  include TestHelper
 
   class TestParent < org.jruby.test.Parent
     attr_accessor :result
@@ -272,14 +274,14 @@ class TestJavaExtension < Test::Unit::TestCase
     assert listing.size >= 0
   end
 
-  class ExtendedClass < org.jruby.test.Abstract
+  class RubyConcrete < org.jruby.test.Abstract
     def protected_method
       "Ruby overrides java!"
     end
   end
 
   def test_overriding_protected_method
-    a = ExtendedClass.new
+    a = RubyConcrete.new
     begin
       assert_equal "Ruby overrides java!", a.call_protected
     rescue Exception => e
@@ -287,6 +289,18 @@ class TestJavaExtension < Test::Unit::TestCase
     end
   end
 
+  class CallAbstractInConstructor < org.jruby.test.Abstract
+    def initialize; super("call protected method in constructor!") end
+
+    def protected_method; "HELLO!" end
+  end
+
+  def test_calling_abstract_method_in_java_constructor
+    return skip('this leaking in super constructor (calling Ruby implemented methods)')
+    a = CallAbstractInConstructor.new
+    assert_equal "HELLO!", a.result
+  end
+
   def test_map_interface_to_array
     hash = {"one"=>"two","three"=>"four"}
     map = java.util.HashMap.new(hash)
