@@ -1,9 +1,12 @@
 require 'test/unit'
+require 'test/test_helper'
 require 'zlib'
 require 'stringio'
 require 'tempfile'
 
 class TestZlib < Test::Unit::TestCase
+  include TestHelper
+
   def teardown;  File.unlink @filename if @filename; end
 
   def test_inflate_deflate
@@ -12,7 +15,7 @@ class TestZlib < Test::Unit::TestCase
       assert_equal(s, Zlib::Inflate.inflate(Zlib::Deflate.deflate(s, level)))
     end
   end
-  
+
   # Zlib::Inflate uses org.jruby.util.ZlibInflate for low-level decompression logic, which is built around
   # java.util.zip.Inflater.
   #
@@ -33,9 +36,9 @@ class TestZlib < Test::Unit::TestCase
       # This is not the ZIP archive itself; rather, it is the compressed representation
       # of a deflated file inside the archive.
       data = "K0ktLlFIzs8tKEotLs7Mz1MoLinKzEvnKqGxOABQSwECFwMUAAIACAAxKng3dpOMHR0AAAB4AAAACAANAAAAAAABAAAApIEAAAAAdGVzdC50eHRVVAUAAz76R0dVeAAAUEsFBgAAAAABAAEAQwAAAFgAAAAAAA=="
-              
+
       inflater = Zlib::Inflate.new(-Zlib::MAX_WBITS)
-      
+
       stream = StringIO.new(data.unpack('m*')[0])
       assert_equal(actual, inflater.inflate(stream.read(nil, '')), "Unexpected result of decompression.")
       assert(inflater.finished?, "Inflater should be finished after inflating all input.")
@@ -43,7 +46,7 @@ class TestZlib < Test::Unit::TestCase
 
   def test_gzip_reader_writer
     @filename = "____temp_zlib_file";
-    
+
     Zlib::GzipWriter.open(@filename) { |z| z.puts 'HEH' }
     Zlib::GzipReader.open(@filename) do |z|
       assert_equal("HEH\n", z.gets)
@@ -51,7 +54,7 @@ class TestZlib < Test::Unit::TestCase
       assert z.eof?
     end
     File.unlink(@filename)
-    
+
     Zlib::GzipWriter.open(@filename) { |z| z.write "HEH\n" }
     Zlib::GzipReader.open(@filename) do |z|
       assert_equal("HEH\n", z.gets)
@@ -59,20 +62,20 @@ class TestZlib < Test::Unit::TestCase
       assert z.eof?
     end
     File.unlink(@filename)
-    
+
 
     z = Zlib::GzipWriter.open(@filename)
     z.puts 'HOH'
     z.puts 'foo|bar'
     z.close
-    
+
     z = Zlib::GzipReader.open(@filename)
     assert_equal("HOH\n", z.gets)
     assert_equal("foo|", z.gets("|"))
     assert_equal("bar\n", z.gets)
     z.close
   end
-  
+
   def test_native_exception_from_zlib_on_broken_header
     corrupt = StringIO.new
     corrupt.write('borkborkbork')
@@ -82,14 +85,14 @@ class TestZlib < Test::Unit::TestCase
     rescue Zlib::GzipReader::Error
     end
   end
-  
+
   def test_deflate_positive_winbits
     d =  Zlib::Deflate.new(Zlib::DEFAULT_COMPRESSION, Zlib::MAX_WBITS)
     d << 'hello'
     res = d.finish
     assert_equal("x\234\313H\315\311\311\a\000\006,\002\025", res)
   end
-  
+
    # negative winbits means no header and no checksum.
    def test_deflate_negative_winbits
      d =  Zlib::Deflate.new(Zlib::DEFAULT_COMPRESSION, -Zlib::MAX_WBITS)
@@ -115,7 +118,7 @@ class TestZlib < Test::Unit::TestCase
 
   def test_wrap
     content = StringIO.new "", "r+"
-    
+
     Zlib::GzipWriter.wrap(content) do |io|
       io.write "hello\nworld\n"
     end
@@ -129,7 +132,7 @@ class TestZlib < Test::Unit::TestCase
     assert gin.eof?
     gin.close
   end
-  
+
   def test_each_line_no_block
     @filename = "____temp_zlib_file";
     Zlib::GzipWriter.open(@filename) { |io| io.write "hello\nworld\n" }
@@ -139,12 +142,12 @@ class TestZlib < Test::Unit::TestCase
       lines << line
     end
     z.close
-    
+
     assert_equal(2, lines.size, lines.inspect)
     assert_equal("hello\n", lines.first)
     assert_equal("world\n", lines.last)
   end
-  
+
   def test_each_line_block
     @filename = "____temp_zlib_file";
     Zlib::GzipWriter.open(@filename) { |io| io.write "hello\nworld\n" }
@@ -156,7 +159,7 @@ class TestZlib < Test::Unit::TestCase
     end
     assert_equal(2, lines.size, lines.inspect)
   end
-  
+
   def test_empty_line
     @filename = "____temp_zlib_file";
     Zlib::GzipWriter.open(@filename) { |io| io.write "hello\nworld\n\ngoodbye\n" }
@@ -436,7 +439,7 @@ class TestZlib < Test::Unit::TestCase
   def test_dup
     d1 = Zlib::Deflate.new
 
-    data = "foo" * 10 
+    data = "foo" * 10
     d1 << data
     d2 = d1.dup
 
@@ -728,7 +731,7 @@ class TestZlibDeflateGzip < Test::Unit::TestCase
     i = Zlib::Inflate.new(15+16)
     "a".upto("z") do |c|
         assert_equal(c, i.inflate(d.deflate(c, Zlib::SYNC_FLUSH)))
-    end 
+    end
     i.inflate(d.finish)
     i.close
     d.close
